@@ -30,7 +30,7 @@ Z = np.empty(X1.shape)
 for (i, j), val in np.ndenumerate(X1):
     x1 = val
     x2 = X2[i, j]
-    p = clf.decision_function([x1, x2])
+    p = clf.decision_function([[x1, x2]])
     Z[i, j] = p[0]
 levels = [-1.0, 0.0, 1.0]
 linestyles = ['dashed', 'solid', 'dashed']
