@@ -10,7 +10,7 @@ For a discussion and comparison of these algorithms, see the
 :ref:`manifold module page <manifold>`
 
 For a similar example, where the methods are applied to a
-sphere dataset, see :ref:`example_manifold_plot_manifold_sphere.py`
+sphere dataset, see :ref:`sphx_glr_auto_examples_manifold_plot_manifold_sphere.py`
 
 Note that the purpose of the MDS is to find a low-dimensional
 representation of the data (here 2D) in which the distances respect well
