@@ -75,10 +75,12 @@ print(__doc__)
 
 import matplotlib.pyplot as plt
 import numpy as np
+from itertools import cycle
+
 from sklearn import svm, datasets
 from sklearn.metrics import precision_recall_curve
 from sklearn.metrics import average_precision_score
-from sklearn.cross_validation import train_test_split
+from sklearn.model_selection import train_test_split
 from sklearn.preprocessing import label_binarize
 from sklearn.multiclass import OneVsRestClassifier
 
@@ -87,6 +89,10 @@ iris = datasets.load_iris()
 X = iris.data
 y = iris.target
 
+# setup plot details
+colors = cycle(['navy', 'turquoise', 'darkorange', 'cornflowerblue', 'teal'])
+lw = 2
+
 # Binarize the output
 y = label_binarize(y, classes=[0, 1, 2])
 n_classes = y.shape[1]
@@ -120,9 +126,11 @@ precision["micro"], recall["micro"], _ = precision_recall_curve(y_test.ravel(),
 average_precision["micro"] = average_precision_score(y_test, y_score,
                                                      average="micro")
 
+
 # Plot Precision-Recall curve
 plt.clf()
-plt.plot(recall[0], precision[0], label='Precision-Recall curve')
+plt.plot(recall[0], precision[0], lw=lw, color='navy',
+         label='Precision-Recall curve')
 plt.xlabel('Recall')
 plt.ylabel('Precision')
 plt.ylim([0.0, 1.05])
@@ -133,11 +141,11 @@ plt.show()
 
 # Plot Precision-Recall curve for each class
 plt.clf()
-plt.plot(recall["micro"], precision["micro"],
+plt.plot(recall["micro"], precision["micro"], color='gold', lw=lw,
          label='micro-average Precision-recall curve (area = {0:0.2f})'
                ''.format(average_precision["micro"]))
-for i in range(n_classes):
-    plt.plot(recall[i], precision[i],
+for i, color in zip(range(n_classes), colors):
+    plt.plot(recall[i], precision[i], color=color, lw=lw,
              label='Precision-recall curve of class {0} (area = {1:0.2f})'
                    ''.format(i, average_precision[i]))
 
