@@ -13,7 +13,7 @@ print(__doc__)
 
 # Authors: Clay Woolam <clay@woolam.org>
 #          Andreas Mueller <amueller@ais.uni-bonn.de>
-# Licence: BSD
+# License: BSD
 
 import numpy as np
 import matplotlib.pyplot as plt
@@ -38,24 +38,24 @@ label_spread.fit(X, labels)
 output_labels = label_spread.transduction_
 plt.figure(figsize=(8.5, 4))
 plt.subplot(1, 2, 1)
-plot_outer_labeled, = plt.plot(X[labels == outer, 0],
-                               X[labels == outer, 1], 'rs')
-plot_unlabeled, = plt.plot(X[labels == -1, 0], X[labels == -1, 1], 'g.')
-plot_inner_labeled, = plt.plot(X[labels == inner, 0],
-                               X[labels == inner, 1], 'bs')
-plt.legend((plot_outer_labeled, plot_inner_labeled, plot_unlabeled),
-           ('Outer Labeled', 'Inner Labeled', 'Unlabeled'), loc='upper left',
-           numpoints=1, shadow=False)
-plt.title("Raw data (2 classes=red and blue)")
+plt.scatter(X[labels == outer, 0], X[labels == outer, 1], color='navy',
+            marker='s', lw=0, label="outer labeled", s=10)
+plt.scatter(X[labels == inner, 0], X[labels == inner, 1], color='c',
+            marker='s', lw=0, label='inner labeled', s=10)
+plt.scatter(X[labels == -1, 0], X[labels == -1, 1], color='darkorange',
+            marker='.', label='unlabeled')
+plt.legend(scatterpoints=1, shadow=False, loc='upper right')
+plt.title("Raw data (2 classes=outer and inner)")
 
 plt.subplot(1, 2, 2)
 output_label_array = np.asarray(output_labels)
 outer_numbers = np.where(output_label_array == outer)[0]
 inner_numbers = np.where(output_label_array == inner)[0]
-plot_outer, = plt.plot(X[outer_numbers, 0], X[outer_numbers, 1], 'rs')
-plot_inner, = plt.plot(X[inner_numbers, 0], X[inner_numbers, 1], 'bs')
-plt.legend((plot_outer, plot_inner), ('Outer Learned', 'Inner Learned'),
-           loc='upper left', numpoints=1, shadow=False)
+plt.scatter(X[outer_numbers, 0], X[outer_numbers, 1], color='navy',
+            marker='s', lw=0, s=10, label="outer learned")
+plt.scatter(X[inner_numbers, 0], X[inner_numbers, 1], color='c',
+            marker='s', lw=0, s=10, label="inner learned")
+plt.legend(scatterpoints=1, shadow=False, loc='upper right')
 plt.title("Labels learned with Label Spreading (KNN)")
 
 plt.subplots_adjust(left=0.07, bottom=0.07, right=0.93, top=0.92)
