@@ -41,13 +41,14 @@ Z = clf.decision_function(np.c_[xx.ravel(), yy.ravel()])
 Z = Z.reshape(xx.shape)
 
 plt.title("Novelty Detection")
-plt.contourf(xx, yy, Z, levels=np.linspace(Z.min(), 0, 7), cmap=plt.cm.Blues_r)
-a = plt.contour(xx, yy, Z, levels=[0], linewidths=2, colors='red')
-plt.contourf(xx, yy, Z, levels=[0, Z.max()], colors='orange')
+plt.contourf(xx, yy, Z, levels=np.linspace(Z.min(), 0, 7), cmap=plt.cm.PuBu)
+a = plt.contour(xx, yy, Z, levels=[0], linewidths=2, colors='darkred')
+plt.contourf(xx, yy, Z, levels=[0, Z.max()], colors='palevioletred')
 
-b1 = plt.scatter(X_train[:, 0], X_train[:, 1], c='white')
-b2 = plt.scatter(X_test[:, 0], X_test[:, 1], c='green')
-c = plt.scatter(X_outliers[:, 0], X_outliers[:, 1], c='red')
+s = 40
+b1 = plt.scatter(X_train[:, 0], X_train[:, 1], c='white', s=s)
+b2 = plt.scatter(X_test[:, 0], X_test[:, 1], c='blueviolet', s=s)
+c = plt.scatter(X_outliers[:, 0], X_outliers[:, 1], c='gold', s=s)
 plt.axis('tight')
 plt.xlim((-5, 5))
 plt.ylim((-5, 5))
