@@ -88,12 +88,11 @@ import numpy as np
 import matplotlib.pyplot as plt
 
 from sklearn.svm import LinearSVC
-from sklearn.cross_validation import ShuffleSplit
-from sklearn.grid_search import GridSearchCV
+from sklearn.model_selection import ShuffleSplit
+from sklearn.model_selection import GridSearchCV
 from sklearn.utils import check_random_state
 from sklearn import datasets
 
-
 rnd = check_random_state(1)
 
 # set up dataset
@@ -117,7 +116,8 @@ clf_sets = [(LinearSVC(penalty='l1', loss='squared_hinge', dual=False,
                        tol=1e-4),
              np.logspace(-4.5, -2, 10), X_2, y_2)]
 
-colors = ['b', 'g', 'r', 'c']
+colors = ['navy', 'cyan', 'darkorange']
+lw = 2
 
 for fignum, (clf, cs, X, y) in enumerate(clf_sets):
     # set up the plot for each regressor
@@ -128,10 +128,10 @@ for fignum, (clf, cs, X, y) in enumerate(clf_sets):
         # To get nice curve, we need a large number of iterations to
         # reduce the variance
         grid = GridSearchCV(clf, refit=False, param_grid=param_grid,
-                            cv=ShuffleSplit(n=n_samples, train_size=train_size,
-                                            n_iter=250, random_state=1))
+                            cv=ShuffleSplit(train_size=train_size,
+                                            n_splits=250, random_state=1))
         grid.fit(X, y)
-        scores = [x[1] for x in grid.grid_scores_]
+        scores = grid.cv_results_['mean_test_score']
 
         scales = [(1, 'No scaling'),
                   ((n_samples * train_size), '1/n_samples'),
@@ -143,7 +143,7 @@ for fignum, (clf, cs, X, y) in enumerate(clf_sets):
             plt.ylabel('CV Score')
             grid_cs = cs * float(scaler)  # scale the C's
             plt.semilogx(grid_cs, scores, label="fraction %.2f" %
-                         train_size)
+                         train_size, color=colors[k], lw=lw)
             plt.title('scaling=%s, penalty=%s, loss=%s' %
                       (name, clf.penalty, clf.loss))
 
