@@ -39,9 +39,9 @@ y_2 = regr_2.predict(X_test)
 
 # Plot the results
 plt.figure()
-plt.scatter(X, y, c="k", label="data")
-plt.plot(X_test, y_1, c="g", label="max_depth=2", linewidth=2)
-plt.plot(X_test, y_2, c="r", label="max_depth=5", linewidth=2)
+plt.scatter(X, y, c="darkorange", label="data")
+plt.plot(X_test, y_1, color="cornflowerblue", label="max_depth=2", linewidth=2)
+plt.plot(X_test, y_2, color="yellowgreen", label="max_depth=5", linewidth=2)
 plt.xlabel("data")
 plt.ylabel("target")
 plt.title("Decision Tree Regression")
