@@ -42,14 +42,15 @@ y_3 = regr_3.predict(X_test)
 
 # Plot the results
 plt.figure()
-plt.scatter(y[:, 0], y[:, 1], c="k", label="data")
-plt.scatter(y_1[:, 0], y_1[:, 1], c="g", label="max_depth=2")
-plt.scatter(y_2[:, 0], y_2[:, 1], c="r", label="max_depth=5")
-plt.scatter(y_3[:, 0], y_3[:, 1], c="b", label="max_depth=8")
+s = 50
+plt.scatter(y[:, 0], y[:, 1], c="navy", s=s, label="data")
+plt.scatter(y_1[:, 0], y_1[:, 1], c="cornflowerblue", s=s, label="max_depth=2")
+plt.scatter(y_2[:, 0], y_2[:, 1], c="c", s=s, label="max_depth=5")
+plt.scatter(y_3[:, 0], y_3[:, 1], c="orange", s=s, label="max_depth=8")
 plt.xlim([-6, 6])
 plt.ylim([-6, 6])
-plt.xlabel("data")
-plt.ylabel("target")
+plt.xlabel("target 1")
+plt.ylabel("target 2")
 plt.title("Multi-output Decision Tree Regression")
 plt.legend()
 plt.show()
