@@ -181,7 +181,7 @@ def test_discretize(seed=8):
     for n_samples in [50, 100, 150, 500]:
         for n_class in range(2, 10):
             # random class labels
-            y_true = random_state.random_integers(0, n_class, n_samples)
+            y_true = random_state.randint(0, n_class + 1, n_samples)
             y_true = np.array(y_true, np.float)
             # noise class assignment matrix
             y_indicator = sparse.coo_matrix((np.ones(n_samples),
