@@ -4,7 +4,7 @@ algorithms, including among others PCA, NMF or ICA. Most of the algorithms of
 this module can be regarded as dimensionality reduction techniques.
 """
 
-from .nmf import NMF, ProjectedGradientNMF
+from .nmf import NMF, ProjectedGradientNMF, non_negative_factorization
 from .pca import PCA, RandomizedPCA
 from .incremental_pca import IncrementalPCA
 from .kernel_pca import KernelPCA
@@ -33,6 +33,7 @@ __all__ = ['DictionaryLearning',
            'dict_learning',
            'dict_learning_online',
            'fastica',
+           'non_negative_factorization',
            'randomized_svd',
            'sparse_encode',
            'FactorAnalysis',
