@@ -17,7 +17,7 @@ Algorithm 21.1
 #         Alexandre Gramfort <alexandre.gramfort@inria.fr>
 #         Denis A. Engemann <d.engemann@fz-juelich.de>
 
-# Licence: BSD3
+# License: BSD3
 
 import warnings
 from math import sqrt, log
@@ -27,9 +27,10 @@ from scipy import linalg
 
 from ..base import BaseEstimator, TransformerMixin
 from ..externals.six.moves import xrange
-from ..utils import array2d, check_arrays, check_random_state
+from ..utils import check_array, check_random_state
 from ..utils.extmath import fast_logdet, fast_dot, randomized_svd, squared_norm
-from ..utils import ConvergenceWarning
+from ..utils.validation import check_is_fitted
+from ..exceptions import ConvergenceWarning
 
 
 class FactorAnalysis(BaseEstimator, TransformerMixin):
@@ -51,6 +52,8 @@ class FactorAnalysis(BaseEstimator, TransformerMixin):
     `loading` matrix, the transformation of the latent variables to the
     observed ones, using expectation-maximization (EM).
 
+    Read more in the :ref:`User Guide <FA>`.
+
     Parameters
     ----------
     n_components : int | None
@@ -68,9 +71,6 @@ class FactorAnalysis(BaseEstimator, TransformerMixin):
     max_iter : int
         Maximum number of iterations.
 
-    verbose : int | bool
-        Print verbose output.
-
     noise_variance_init : None | array, shape=(n_features,)
         The initial guess of the noise variance for each feature.
         If None, it defaults to np.ones(n_features)
@@ -94,15 +94,18 @@ class FactorAnalysis(BaseEstimator, TransformerMixin):
 
     Attributes
     ----------
-    `components_` : array, [n_components, n_features]
+    components_ : array, [n_components, n_features]
         Components with maximum variance.
 
-    `loglike_` : list, [n_iterations]
+    loglike_ : list, [n_iterations]
         The log likelihood at each iteration.
 
-    `noise_variance_` : array, shape=(n_features,)
+    noise_variance_ : array, shape=(n_features,)
         The estimated noise variance for each feature.
 
+    n_iter_ : int
+        Number of iterations run.
+
     References
     ----------
     .. David Barber, Bayesian Reasoning and Machine Learning,
@@ -121,7 +124,7 @@ class FactorAnalysis(BaseEstimator, TransformerMixin):
         non-Gaussian latent variables.
     """
     def __init__(self, n_components=None, tol=1e-2, copy=True, max_iter=1000,
-                 verbose=0, noise_variance_init=None, svd_method='randomized',
+                 noise_variance_init=None, svd_method='randomized',
                  iterated_power=3, random_state=0):
         self.n_components = n_components
         self.copy = copy
@@ -131,13 +134,7 @@ class FactorAnalysis(BaseEstimator, TransformerMixin):
             raise ValueError('SVD method %s is not supported. Please consider'
                              ' the documentation' % svd_method)
         self.svd_method = svd_method
-        if verbose:
-            warnings.warn('The `verbose` parameter has been deprecated and '
-                          'will be removed in 0.16. To reduce verbosity '
-                          'silence Python warnings instead.',
-                          DeprecationWarning)
 
-        self.verbose = verbose
         self.noise_variance_init = noise_variance_init
         self.iterated_power = iterated_power
         self.random_state = random_state
@@ -154,8 +151,7 @@ class FactorAnalysis(BaseEstimator, TransformerMixin):
         -------
         self
         """
-        X = array2d(check_arrays(X, copy=self.copy, sparse_format='dense',
-                    dtype=np.float)[0])
+        X = check_array(X, copy=self.copy, dtype=np.float64)
 
         n_samples, n_features = X.shape
         n_components = self.n_components
@@ -230,6 +226,7 @@ class FactorAnalysis(BaseEstimator, TransformerMixin):
         self.components_ = W
         self.noise_variance_ = psi
         self.loglike_ = loglike
+        self.n_iter_ = i + 1
         return self
 
     def transform(self, X):
@@ -248,7 +245,9 @@ class FactorAnalysis(BaseEstimator, TransformerMixin):
         X_new : array-like, shape (n_samples, n_components)
             The latent variables of X.
         """
-        X = array2d(X)
+        check_is_fitted(self, 'components_')
+
+        X = check_array(X)
         Ih = np.eye(len(self.components_))
 
         X_transformed = X - self.mean_
@@ -270,6 +269,8 @@ class FactorAnalysis(BaseEstimator, TransformerMixin):
         cov : array, shape (n_features, n_features)
             Estimated covariance of data.
         """
+        check_is_fitted(self, 'components_')
+
         cov = np.dot(self.components_.T, self.components_)
         cov.flat[::len(cov) + 1] += self.noise_variance_  # modify diag inplace
         return cov
@@ -282,6 +283,8 @@ class FactorAnalysis(BaseEstimator, TransformerMixin):
         precision : array, shape (n_features, n_features)
             Estimated precision of data.
         """
+        check_is_fitted(self, 'components_')
+
         n_features = self.components_.shape[1]
 
         # handle corner cases first
@@ -314,6 +317,8 @@ class FactorAnalysis(BaseEstimator, TransformerMixin):
         ll: array, shape (n_samples,)
             Log-likelihood of each sample under the current model
         """
+        check_is_fitted(self, 'components_')
+
         Xr = X - self.mean_
         precision = self.get_precision()
         n_features = X.shape[1]
