@@ -1,12 +1,17 @@
 import numpy as np
 
+from sklearn.exceptions import ConvergenceWarning
+
+from sklearn.utils import check_array
+
 from sklearn.utils.testing import assert_array_almost_equal
 from sklearn.utils.testing import assert_array_equal
 from sklearn.utils.testing import assert_equal
-from sklearn.utils.testing import SkipTest
 from sklearn.utils.testing import assert_true
 from sklearn.utils.testing import assert_less
 from sklearn.utils.testing import assert_raises
+from sklearn.utils.testing import ignore_warnings
+from sklearn.utils.testing import TempMemmap
 
 from sklearn.decomposition import DictionaryLearning
 from sklearn.decomposition import MiniBatchDictionaryLearning
@@ -47,15 +52,40 @@ def test_dict_learning_reconstruction():
     # nonzero atoms is right.
 
 
+def test_dict_learning_reconstruction_parallel():
+    # regression test that parallel reconstruction works with n_jobs=-1
+    n_components = 12
+    dico = DictionaryLearning(n_components, transform_algorithm='omp',
+                              transform_alpha=0.001, random_state=0, n_jobs=-1)
+    code = dico.fit(X).transform(X)
+    assert_array_almost_equal(np.dot(code, dico.components_), X)
+
+    dico.set_params(transform_algorithm='lasso_lars')
+    code = dico.transform(X)
+    assert_array_almost_equal(np.dot(code, dico.components_), X, decimal=2)
+
+
+def test_dict_learning_lassocd_readonly_data():
+    n_components = 12
+    with TempMemmap(X) as X_read_only:
+        dico = DictionaryLearning(n_components, transform_algorithm='lasso_cd',
+                                  transform_alpha=0.001, random_state=0,
+                                  n_jobs=-1)
+        with ignore_warnings(category=ConvergenceWarning):
+            code = dico.fit(X_read_only).transform(X_read_only)
+        assert_array_almost_equal(np.dot(code, dico.components_), X_read_only,
+                                  decimal=2)
+
+
 def test_dict_learning_nonzero_coefs():
     n_components = 4
     dico = DictionaryLearning(n_components, transform_algorithm='lars',
                               transform_n_nonzero_coefs=3, random_state=0)
-    code = dico.fit(X).transform(X[1])
+    code = dico.fit(X).transform(X[np.newaxis, 1])
     assert_true(len(np.flatnonzero(code)) == 3)
 
     dico.set_params(transform_algorithm='omp')
-    code = dico.transform(X[1])
+    code = dico.transform(X[np.newaxis, 1])
     assert_equal(len(np.flatnonzero(code)), 3)
 
 
@@ -92,19 +122,23 @@ def test_dict_learning_online_verbosity():
     # test verbosity
     from sklearn.externals.six.moves import cStringIO as StringIO
     import sys
+
     old_stdout = sys.stdout
-    sys.stdout = StringIO()
-    dico = MiniBatchDictionaryLearning(n_components, n_iter=20, verbose=1,
-                                       random_state=0)
-    dico.fit(X)
-    dico = MiniBatchDictionaryLearning(n_components, n_iter=20, verbose=2,
-                                       random_state=0)
-    dico.fit(X)
-    dict_learning_online(X, n_components=n_components, alpha=1, verbose=1,
-                         random_state=0)
-    dict_learning_online(X, n_components=n_components, alpha=1, verbose=2,
-                         random_state=0)
-    sys.stdout = old_stdout
+    try:
+        sys.stdout = StringIO()
+        dico = MiniBatchDictionaryLearning(n_components, n_iter=20, verbose=1,
+                                           random_state=0)
+        dico.fit(X)
+        dico = MiniBatchDictionaryLearning(n_components, n_iter=20, verbose=2,
+                                           random_state=0)
+        dico.fit(X)
+        dict_learning_online(X, n_components=n_components, alpha=1, verbose=1,
+                             random_state=0)
+        dict_learning_online(X, n_components=n_components, alpha=1, verbose=2,
+                             random_state=0)
+    finally:
+        sys.stdout = old_stdout
+
     assert_true(dico.components_.shape == (n_components, n_features))
 
 
@@ -132,23 +166,25 @@ def test_dict_learning_online_initialization():
 
 
 def test_dict_learning_online_partial_fit():
-    # this test was not actually passing before!
-    raise SkipTest
     n_components = 12
     rng = np.random.RandomState(0)
     V = rng.randn(n_components, n_features)  # random init
     V /= np.sum(V ** 2, axis=1)[:, np.newaxis]
-    dico1 = MiniBatchDictionaryLearning(n_components, n_iter=10, batch_size=1,
-                                        shuffle=False, dict_init=V,
+    dict1 = MiniBatchDictionaryLearning(n_components, n_iter=10 * len(X),
+                                        batch_size=1,
+                                        alpha=1, shuffle=False, dict_init=V,
                                         random_state=0).fit(X)
-    dico2 = MiniBatchDictionaryLearning(n_components, n_iter=1, dict_init=V,
+    dict2 = MiniBatchDictionaryLearning(n_components, alpha=1,
+                                        n_iter=1, dict_init=V,
                                         random_state=0)
-    for ii, sample in enumerate(X):
-        dico2.partial_fit(sample, iter_offset=ii * dico2.n_iter)
-        # if ii == 1: break
-    assert_true(not np.all(sparse_encode(X, dico1.components_, alpha=100) ==
+    for i in range(10):
+        for sample in X:
+            dict2.partial_fit(sample[np.newaxis, :])
+
+    assert_true(not np.all(sparse_encode(X, dict1.components_, alpha=1) ==
                            0))
-    assert_array_equal(dico1.components_, dico2.components_)
+    assert_array_almost_equal(dict1.components_, dict2.components_,
+                              decimal=2)
 
 
 def test_sparse_encode_shapes():
@@ -161,6 +197,18 @@ def test_sparse_encode_shapes():
         assert_equal(code.shape, (n_samples, n_components))
 
 
+def test_sparse_encode_input():
+    n_components = 100
+    rng = np.random.RandomState(0)
+    V = rng.randn(n_components, n_features)  # random init
+    V /= np.sum(V ** 2, axis=1)[:, np.newaxis]
+    Xf = check_array(X, order='F')
+    for algo in ('lasso_lars', 'lasso_cd', 'lars', 'omp', 'threshold'):
+        a = sparse_encode(X, V, algorithm=algo)
+        b = sparse_encode(Xf, V, algorithm=algo)
+        assert_array_almost_equal(a, b)
+
+
 def test_sparse_encode_error():
     n_components = 12
     rng = np.random.RandomState(0)
@@ -171,6 +219,15 @@ def test_sparse_encode_error():
     assert_less(np.sqrt(np.sum((np.dot(code, V) - X) ** 2)), 0.1)
 
 
+def test_sparse_encode_error_default_sparsity():
+    rng = np.random.RandomState(0)
+    X = rng.randn(100, 64)
+    D = rng.randn(2, 64)
+    code = ignore_warnings(sparse_encode)(X, D, algorithm='omp',
+                                          n_nonzero_coefs=None)
+    assert_equal(code.shape, (100, 2))
+
+
 def test_unknown_method():
     n_components = 12
     rng = np.random.RandomState(0)
