@@ -5,7 +5,7 @@ regression.
 
 The module structure is the following:
 
-- The ``BaseAdaBoost`` base class implements a common ``fit`` method
+- The ``BaseWeightBoosting`` base class implements a common ``fit`` method
   for all the estimators in the module. Regression and classification
   only differ from each other in the loss function that is optimized.
 
@@ -19,27 +19,27 @@ The module structure is the following:
 # Authors: Noel Dawe <noel@dawe.me>
 #          Gilles Louppe <g.louppe@gmail.com>
 #          Hamzeh Alsalhi <ha258@cornell.edu>
-# Licence: BSD 3 clause
+#          Arnaud Joly <arnaud.v.joly@gmail.com>
+#
+# License: BSD 3 clause
 
 from abc import ABCMeta, abstractmethod
 
 import numpy as np
 from numpy.core.umath_tests import inner1d
 
-from scipy.sparse import coo_matrix
-from scipy.sparse import issparse
-
 from .base import BaseEnsemble
-from ..base import ClassifierMixin, RegressorMixin
+from ..base import ClassifierMixin, RegressorMixin, is_regressor
 from ..externals import six
-from ..externals.six.moves import xrange, zip
+from ..externals.six.moves import zip
+from ..externals.six.moves import xrange as range
 from .forest import BaseForest
 from ..tree import DecisionTreeClassifier, DecisionTreeRegressor
 from ..tree.tree import BaseDecisionTree
 from ..tree._tree import DTYPE
-from ..utils import array2d, check_arrays, check_random_state, column_or_1d
-from ..utils import safe_asarray
+from ..utils import check_array, check_X_y, check_random_state
 from ..metrics import accuracy_score, r2_score
+from sklearn.utils.validation import has_fit_parameter, check_is_fitted
 
 __all__ = [
     'AdaBoostClassifier',
@@ -98,24 +98,21 @@ class BaseWeightBoosting(six.with_metaclass(ABCMeta, BaseEnsemble)):
         if self.learning_rate <= 0:
             raise ValueError("learning_rate must be greater than zero")
 
-        # Check data
-        if isinstance(X, coo_matrix):
-            X = X.tocsr()
-        X = safe_asarray(X)
+        if (self.base_estimator is None or
+                isinstance(self.base_estimator, (BaseDecisionTree,
+                                                 BaseForest))):
+            dtype = DTYPE
+            accept_sparse = 'csc'
+        else:
+            dtype = None
+            accept_sparse = ['csr', 'csc']
 
-        if (X.ndim != 2 and not issparse(X)):
-            X = array2d(X)
-
-        if(self.base_estimator is None or
-           isinstance(self.base_estimator, (BaseDecisionTree, BaseForest))):
-            X, = check_arrays(X, dtype=DTYPE)
-        X, y = check_arrays(X, y, check_ccontiguous=True)
-
-        y = column_or_1d(y, warn=True)
+        X, y = check_X_y(X, y, accept_sparse=accept_sparse, dtype=dtype,
+                         y_numeric=is_regressor(self))
 
         if sample_weight is None:
             # Initialize weights to 1 / n_samples
-            sample_weight = np.empty(X.shape[0], dtype=np.float)
+            sample_weight = np.empty(X.shape[0], dtype=np.float64)
             sample_weight[:] = 1. / X.shape[0]
         else:
             # Normalize existing weights
@@ -132,15 +129,18 @@ class BaseWeightBoosting(six.with_metaclass(ABCMeta, BaseEnsemble)):
 
         # Clear any previous fit results
         self.estimators_ = []
-        self.estimator_weights_ = np.zeros(self.n_estimators, dtype=np.float)
-        self.estimator_errors_ = np.ones(self.n_estimators, dtype=np.float)
+        self.estimator_weights_ = np.zeros(self.n_estimators, dtype=np.float64)
+        self.estimator_errors_ = np.ones(self.n_estimators, dtype=np.float64)
 
-        for iboost in xrange(self.n_estimators):
+        random_state = check_random_state(self.random_state)
+
+        for iboost in range(self.n_estimators):
             # Boosting step
             sample_weight, estimator_weight, estimator_error = self._boost(
                 iboost,
                 X, y,
-                sample_weight)
+                sample_weight,
+                random_state)
 
             # Early termination
             if sample_weight is None:
@@ -165,15 +165,11 @@ class BaseWeightBoosting(six.with_metaclass(ABCMeta, BaseEnsemble)):
 
         return self
 
-    def _check_fitted(self):
-        if not hasattr(self, "estimators_"):
-            raise ValueError("call fit first")
-
     @abstractmethod
-    def _boost(self, iboost, X, y, sample_weight):
+    def _boost(self, iboost, X, y, sample_weight, random_state):
         """Implement a single boost.
 
-        Warning: This method needs to be overriden by subclasses.
+        Warning: This method needs to be overridden by subclasses.
 
         Parameters
         ----------
@@ -190,6 +186,9 @@ class BaseWeightBoosting(six.with_metaclass(ABCMeta, BaseEnsemble)):
         sample_weight : array-like of shape = [n_samples]
             The current sample weights.
 
+        random_state : numpy.RandomState
+            The current random number generator
+
         Returns
         -------
         sample_weight : array-like of shape = [n_samples] or None
@@ -260,6 +259,18 @@ class BaseWeightBoosting(six.with_metaclass(ABCMeta, BaseEnsemble)):
                 "since base_estimator does not have a "
                 "feature_importances_ attribute")
 
+    def _validate_X_predict(self, X):
+        """Ensure that X is in the proper format"""
+        if (self.base_estimator is None or
+                isinstance(self.base_estimator,
+                           (BaseDecisionTree, BaseForest))):
+            X = check_array(X, accept_sparse='csr', dtype=DTYPE)
+
+        else:
+            X = check_array(X, accept_sparse=['csr', 'csc', 'coo'])
+
+        return X
+
 
 def _samme_proba(estimator, n_classes, X):
     """Calculate algorithm 4, step 2, equation c) of Zhu et al [1].
@@ -274,7 +285,7 @@ def _samme_proba(estimator, n_classes, X):
     # Displace zero probabilities so the log is defined.
     # Also fix negative elements which may occur with
     # negative sample weights.
-    proba[proba <= 0] = 1e-5
+    proba[proba < np.finfo(proba.dtype).eps] = np.finfo(proba.dtype).eps
     log_proba = np.log(proba)
 
     return (n_classes - 1) * (log_proba - (1. / n_classes)
@@ -292,6 +303,8 @@ class AdaBoostClassifier(BaseWeightBoosting, ClassifierMixin):
 
     This class implements the algorithm known as AdaBoost-SAMME [2].
 
+    Read more in the :ref:`User Guide <adaboost>`.
+
     Parameters
     ----------
     base_estimator : object, optional (default=DecisionTreeClassifier)
@@ -323,23 +336,23 @@ class AdaBoostClassifier(BaseWeightBoosting, ClassifierMixin):
 
     Attributes
     ----------
-    `estimators_` : list of classifiers
+    estimators_ : list of classifiers
         The collection of fitted sub-estimators.
 
-    `classes_` : array of shape = [n_classes]
+    classes_ : array of shape = [n_classes]
         The classes labels.
 
-    `n_classes_` : int
+    n_classes_ : int
         The number of classes.
 
-    `estimator_weights_` : array of floats
+    estimator_weights_ : array of floats
         Weights for each estimator in the boosted ensemble.
 
-    `estimator_errors_` : array of floats
+    estimator_errors_ : array of floats
         Classification error for each estimator in the boosted
         ensemble.
 
-    `feature_importances_` : array of shape = [n_features]
+    feature_importances_ : array of shape = [n_features]
         The feature importances if supported by the ``base_estimator``.
 
     See also
@@ -392,8 +405,7 @@ class AdaBoostClassifier(BaseWeightBoosting, ClassifierMixin):
         """
         # Check that algorithm is supported
         if self.algorithm not in ('SAMME', 'SAMME.R'):
-            raise ValueError("algorithm %s is not supported"
-                             % self.algorithm)
+            raise ValueError("algorithm %s is not supported" % self.algorithm)
 
         # Fit
         return super(AdaBoostClassifier, self).fit(X, y, sample_weight)
@@ -412,8 +424,11 @@ class AdaBoostClassifier(BaseWeightBoosting, ClassifierMixin):
                     "probabilities with a predict_proba method.\n"
                     "Please change the base estimator or set "
                     "algorithm='SAMME' instead.")
+        if not has_fit_parameter(self.base_estimator_, "sample_weight"):
+            raise ValueError("%s doesn't support sample_weight."
+                             % self.base_estimator_.__class__.__name__)
 
-    def _boost(self, iboost, X, y, sample_weight):
+    def _boost(self, iboost, X, y, sample_weight, random_state):
         """Implement a single boost.
 
         Perform a single boost according to the real multi-class SAMME.R
@@ -435,6 +450,9 @@ class AdaBoostClassifier(BaseWeightBoosting, ClassifierMixin):
         sample_weight : array-like of shape = [n_samples]
             The current sample weights.
 
+        random_state : numpy.RandomState
+            The current random number generator
+
         Returns
         -------
         sample_weight : array-like of shape = [n_samples] or None
@@ -450,19 +468,15 @@ class AdaBoostClassifier(BaseWeightBoosting, ClassifierMixin):
             If None then boosting has terminated early.
         """
         if self.algorithm == 'SAMME.R':
-            return self._boost_real(iboost, X, y, sample_weight)
+            return self._boost_real(iboost, X, y, sample_weight, random_state)
 
         else:  # elif self.algorithm == "SAMME":
-            return self._boost_discrete(iboost, X, y, sample_weight)
+            return self._boost_discrete(iboost, X, y, sample_weight,
+                                        random_state)
 
-    def _boost_real(self, iboost, X, y, sample_weight):
+    def _boost_real(self, iboost, X, y, sample_weight, random_state):
         """Implement a single boost using the SAMME.R real algorithm."""
-        estimator = self._make_estimator()
-
-        try:
-            estimator.set_params(random_state=self.random_state)
-        except ValueError:
-            pass
+        estimator = self._make_estimator(random_state=random_state)
 
         estimator.fit(X, y, sample_weight=sample_weight)
 
@@ -501,7 +515,8 @@ class AdaBoostClassifier(BaseWeightBoosting, ClassifierMixin):
         # Displace zero probabilities so the log is defined.
         # Also fix negative elements which may occur with
         # negative sample weights.
-        y_predict_proba[y_predict_proba <= 0] = 1e-5
+        proba = y_predict_proba  # alias for readability
+        proba[proba < np.finfo(proba.dtype).eps] = np.finfo(proba.dtype).eps
 
         # Boost weight using multi-class AdaBoost SAMME.R alg
         estimator_weight = (-1. * self.learning_rate
@@ -517,14 +532,9 @@ class AdaBoostClassifier(BaseWeightBoosting, ClassifierMixin):
 
         return sample_weight, 1., estimator_error
 
-    def _boost_discrete(self, iboost, X, y, sample_weight):
+    def _boost_discrete(self, iboost, X, y, sample_weight, random_state):
         """Implement a single boost using the SAMME discrete algorithm."""
-        estimator = self._make_estimator()
-
-        try:
-            estimator.set_params(random_state=self.random_state)
-        except ValueError:
-            pass
+        estimator = self._make_estimator(random_state=random_state)
 
         estimator.fit(X, y, sample_weight=sample_weight)
 
@@ -550,6 +560,10 @@ class AdaBoostClassifier(BaseWeightBoosting, ClassifierMixin):
         # Stop if the error is at least as bad as random guessing
         if estimator_error >= 1. - (1. / n_classes):
             self.estimators_.pop(-1)
+            if len(self.estimators_) == 0:
+                raise ValueError('BaseClassifier in AdaBoostClassifier '
+                                 'ensemble is worse than random, ensemble '
+                                 'can not be fit.')
             return None, None, None
 
         # Boost weight using multi-class AdaBoost SAMME alg
@@ -641,8 +655,8 @@ class AdaBoostClassifier(BaseWeightBoosting, ClassifierMixin):
             values closer to -1 or 1 mean more like the first or second
             class in ``classes_``, respectively.
         """
-        self._check_fitted()
-        X = safe_asarray(X)
+        check_is_fitted(self, "n_classes_")
+        X = self._validate_X_predict(X)
 
         n_classes = self.n_classes_
         classes = self.classes_[:, np.newaxis]
@@ -685,8 +699,8 @@ class AdaBoostClassifier(BaseWeightBoosting, ClassifierMixin):
             values closer to -1 or 1 mean more like the first or second
             class in ``classes_``, respectively.
         """
-        self._check_fitted()
-        X = safe_asarray(X)
+        check_is_fitted(self, "n_classes_")
+        X = self._validate_X_predict(X)
 
         n_classes = self.n_classes_
         classes = self.classes_[:, np.newaxis]
@@ -735,8 +749,10 @@ class AdaBoostClassifier(BaseWeightBoosting, ClassifierMixin):
             The class probabilities of the input samples. The order of
             outputs is the same of that of the `classes_` attribute.
         """
+        check_is_fitted(self, "n_classes_")
+
         n_classes = self.n_classes_
-        X = safe_asarray(X)
+        X = self._validate_X_predict(X)
 
         if self.algorithm == 'SAMME.R':
             # The weights are all 1. for SAMME.R
@@ -779,6 +795,8 @@ class AdaBoostClassifier(BaseWeightBoosting, ClassifierMixin):
             The class probabilities of the input samples. The order of
             outputs is the same of that of the `classes_` attribute.
         """
+        X = self._validate_X_predict(X)
+
         n_classes = self.n_classes_
         proba = None
         norm = 0.
@@ -838,6 +856,8 @@ class AdaBoostRegressor(BaseWeightBoosting, RegressorMixin):
 
     This class implements the algorithm known as AdaBoost.R2 [2].
 
+    Read more in the :ref:`User Guide <adaboost>`.
+
     Parameters
     ----------
     base_estimator : object, optional (default=DecisionTreeRegressor)
@@ -865,16 +885,16 @@ class AdaBoostRegressor(BaseWeightBoosting, RegressorMixin):
 
     Attributes
     ----------
-    `estimators_` : list of classifiers
+    estimators_ : list of classifiers
         The collection of fitted sub-estimators.
 
-    `estimator_weights_` : array of floats
+    estimator_weights_ : array of floats
         Weights for each estimator in the boosted ensemble.
 
-    `estimator_errors_` : array of floats
+    estimator_errors_ : array of floats
         Regression error for each estimator in the boosted ensemble.
 
-    `feature_importances_` : array of shape = [n_features]
+    feature_importances_ : array of shape = [n_features]
         The feature importances if supported by the ``base_estimator``.
 
     See also
@@ -939,7 +959,7 @@ class AdaBoostRegressor(BaseWeightBoosting, RegressorMixin):
         super(AdaBoostRegressor, self)._validate_estimator(
             default=DecisionTreeRegressor(max_depth=3))
 
-    def _boost(self, iboost, X, y, sample_weight):
+    def _boost(self, iboost, X, y, sample_weight, random_state):
         """Implement a single boost for regression
 
         Perform a single boost according to the AdaBoost.R2 algorithm and
@@ -961,6 +981,9 @@ class AdaBoostRegressor(BaseWeightBoosting, RegressorMixin):
         sample_weight : array-like of shape = [n_samples]
             The current sample weights.
 
+        random_state : numpy.RandomState
+            The current random number generator
+
         Returns
         -------
         sample_weight : array-like of shape = [n_samples] or None
@@ -975,20 +998,13 @@ class AdaBoostRegressor(BaseWeightBoosting, RegressorMixin):
             The regression error for the current boost.
             If None then boosting has terminated early.
         """
-        estimator = self._make_estimator()
-
-        try:
-            estimator.set_params(random_state=self.random_state)
-        except ValueError:
-            pass
-
-        generator = check_random_state(self.random_state)
+        estimator = self._make_estimator(random_state=random_state)
 
         # Weighted sampling of the training set with replacement
         # For NumPy >= 1.7.0 use np.random.choice
         cdf = sample_weight.cumsum()
         cdf /= cdf[-1]
-        uniform_samples = generator.random_sample(X.shape[0])
+        uniform_samples = random_state.random_sample(X.shape[0])
         bootstrap_idx = cdf.searchsorted(uniform_samples, side='right')
         # searchsorted returns a scalar
         bootstrap_idx = np.array(bootstrap_idx, copy=False)
@@ -1069,8 +1085,8 @@ class AdaBoostRegressor(BaseWeightBoosting, RegressorMixin):
         y : array of shape = [n_samples]
             The predicted regression values.
         """
-        self._check_fitted()
-        X = safe_asarray(X)
+        check_is_fitted(self, "estimator_weights_")
+        X = self._validate_X_predict(X)
 
         return self._get_median_predict(X, len(self.estimators_))
 
@@ -1095,8 +1111,8 @@ class AdaBoostRegressor(BaseWeightBoosting, RegressorMixin):
         y : generator of array, shape = [n_samples]
             The predicted regression values.
         """
-        self._check_fitted()
-        X = safe_asarray(X)
+        check_is_fitted(self, "estimator_weights_")
+        X = self._validate_X_predict(X)
 
         for i, _ in enumerate(self.estimators_, 1):
             yield self._get_median_predict(X, limit=i)
