@@ -1,7 +1,6 @@
 """
 Testing Recursive feature elimination
 """
-
 import numpy as np
 from numpy.testing import assert_array_almost_equal, assert_array_equal
 from nose.tools import assert_equal, assert_true
@@ -11,17 +10,20 @@ from sklearn.feature_selection.rfe import RFE, RFECV
 from sklearn.datasets import load_iris, make_friedman1
 from sklearn.metrics import zero_one_loss
 from sklearn.svm import SVC, SVR
+from sklearn.ensemble import RandomForestClassifier
+from sklearn.model_selection import cross_val_score
 
 from sklearn.utils import check_random_state
 from sklearn.utils.testing import ignore_warnings
-from sklearn.utils.testing import assert_warns_message
+from sklearn.utils.testing import assert_greater
 
 from sklearn.metrics import make_scorer
 from sklearn.metrics import get_scorer
 
+
 class MockClassifier(object):
     """
-    Dummy classifier to test recursive feature ellimination
+    Dummy classifier to test recursive feature elimination
     """
 
     def __init__(self, foo_param=0):
@@ -53,42 +55,24 @@ class MockClassifier(object):
         return self
 
 
-def test_rfe_set_params():
+def test_rfe_features_importance():
     generator = check_random_state(0)
     iris = load_iris()
     X = np.c_[iris.data, generator.normal(size=(len(iris.data), 6))]
     y = iris.target
-    clf = SVC(kernel="linear")
+
+    clf = RandomForestClassifier(n_estimators=20,
+                                 random_state=generator, max_depth=2)
     rfe = RFE(estimator=clf, n_features_to_select=4, step=0.1)
-    y_pred = rfe.fit(X, y).predict(X)
+    rfe.fit(X, y)
+    assert_equal(len(rfe.ranking_), X.shape[1])
 
-    clf = SVC()
-    rfe = RFE(estimator=clf, n_features_to_select=4, step=0.1,
-              estimator_params={'kernel': 'linear'})
-    y_pred2 = rfe.fit(X, y).predict(X)
-    assert_array_equal(y_pred, y_pred2)
+    clf_svc = SVC(kernel="linear")
+    rfe_svc = RFE(estimator=clf_svc, n_features_to_select=4, step=0.1)
+    rfe_svc.fit(X, y)
 
-
-def test_rfe_deprecation_estimator_params():
-    deprecation_message = ("The parameter 'estimator_params' is deprecated as of version 0.16 "
-                           "and will be removed in 0.18. The parameter is no longer "
-                           "necessary because the value is set via the estimator initialisation "
-                           "or set_params function.")
-    generator = check_random_state(0)
-    iris = load_iris()
-    X = np.c_[iris.data, generator.normal(size=(len(iris.data), 6))]
-    y = iris.target
-    assert_warns_message(DeprecationWarning, deprecation_message,
-                         RFE(estimator=SVC(), n_features_to_select=4, step=0.1,
-                             estimator_params={'kernel': 'linear'}).fit,
-                         X=X,
-                         y=y)
-
-    assert_warns_message(DeprecationWarning, deprecation_message,
-                         RFECV(estimator=SVC(), step=1, cv=5,
-                               estimator_params={'kernel': 'linear'}).fit,
-                         X=X,
-                         y=y)
+    # Check if the supports are equal
+    assert_array_equal(rfe.get_support(), rfe_svc.get_support())
 
 
 def test_rfe():
@@ -213,8 +197,16 @@ def test_rfecv_mockclassifier():
     assert_equal(len(rfecv.ranking_), X.shape[1])
 
 
-def test_rfe_min_step():
+def test_rfe_estimator_tags():
+    rfe = RFE(SVC(kernel='linear'))
+    assert_equal(rfe._estimator_type, "classifier")
+    # make sure that cross-validation is stratified
+    iris = load_iris()
+    score = cross_val_score(rfe, iris.data, iris.target)
+    assert_greater(score.min(), .7)
 
+
+def test_rfe_min_step():
     n_features = 10
     X, y = make_friedman1(n_samples=50, n_features=n_features, random_state=0)
     n_samples, n_features = X.shape
@@ -222,15 +214,92 @@ def test_rfe_min_step():
 
     # Test when floor(step * n_features) <= 0
     selector = RFE(estimator, step=0.01)
-    sel = selector.fit(X,y)
+    sel = selector.fit(X, y)
     assert_equal(sel.support_.sum(), n_features // 2)
 
     # Test when step is between (0,1) and floor(step * n_features) > 0
     selector = RFE(estimator, step=0.20)
-    sel = selector.fit(X,y)
+    sel = selector.fit(X, y)
     assert_equal(sel.support_.sum(), n_features // 2)
 
     # Test when step is an integer
     selector = RFE(estimator, step=5)
-    sel = selector.fit(X,y)
+    sel = selector.fit(X, y)
     assert_equal(sel.support_.sum(), n_features // 2)
+
+
+def test_number_of_subsets_of_features():
+    # In RFE, 'number_of_subsets_of_features'
+    # = the number of iterations in '_fit'
+    # = max(ranking_)
+    # = 1 + (n_features + step - n_features_to_select - 1) // step
+    # After optimization #4534, this number
+    # = 1 + np.ceil((n_features - n_features_to_select) / float(step))
+    # This test case is to test their equivalence, refer to #4534 and #3824
+
+    def formula1(n_features, n_features_to_select, step):
+        return 1 + ((n_features + step - n_features_to_select - 1) // step)
+
+    def formula2(n_features, n_features_to_select, step):
+        return 1 + np.ceil((n_features - n_features_to_select) / float(step))
+
+    # RFE
+    # Case 1, n_features - n_features_to_select is divisible by step
+    # Case 2, n_features - n_features_to_select is not divisible by step
+    n_features_list = [11, 11]
+    n_features_to_select_list = [3, 3]
+    step_list = [2, 3]
+    for n_features, n_features_to_select, step in zip(
+            n_features_list, n_features_to_select_list, step_list):
+        generator = check_random_state(43)
+        X = generator.normal(size=(100, n_features))
+        y = generator.rand(100).round()
+        rfe = RFE(estimator=SVC(kernel="linear"),
+                  n_features_to_select=n_features_to_select, step=step)
+        rfe.fit(X, y)
+        # this number also equals to the maximum of ranking_
+        assert_equal(np.max(rfe.ranking_),
+                     formula1(n_features, n_features_to_select, step))
+        assert_equal(np.max(rfe.ranking_),
+                     formula2(n_features, n_features_to_select, step))
+
+    # In RFECV, 'fit' calls 'RFE._fit'
+    # 'number_of_subsets_of_features' of RFE
+    # = the size of 'grid_scores' of RFECV
+    # = the number of iterations of the for loop before optimization #4534
+
+    # RFECV, n_features_to_select = 1
+    # Case 1, n_features - 1 is divisible by step
+    # Case 2, n_features - 1 is not divisible by step
+
+    n_features_to_select = 1
+    n_features_list = [11, 10]
+    step_list = [2, 2]
+    for n_features, step in zip(n_features_list, step_list):
+        generator = check_random_state(43)
+        X = generator.normal(size=(100, n_features))
+        y = generator.rand(100).round()
+        rfecv = RFECV(estimator=SVC(kernel="linear"), step=step, cv=5)
+        rfecv.fit(X, y)
+
+        assert_equal(rfecv.grid_scores_.shape[0],
+                     formula1(n_features, n_features_to_select, step))
+        assert_equal(rfecv.grid_scores_.shape[0],
+                     formula2(n_features, n_features_to_select, step))
+
+
+def test_rfe_cv_n_jobs():
+    generator = check_random_state(0)
+    iris = load_iris()
+    X = np.c_[iris.data, generator.normal(size=(len(iris.data), 6))]
+    y = iris.target
+
+    rfecv = RFECV(estimator=SVC(kernel='linear'))
+    rfecv.fit(X, y)
+    rfecv_ranking = rfecv.ranking_
+    rfecv_grid_scores = rfecv.grid_scores_
+
+    rfecv.set_params(n_jobs=2)
+    rfecv.fit(X, y)
+    assert_array_almost_equal(rfecv.ranking_, rfecv_ranking)
+    assert_array_almost_equal(rfecv.grid_scores_, rfecv_grid_scores)
