@@ -2,7 +2,7 @@
 
 # Author: Vincent Dubourg <vincent.dubourg@gmail.com>
 #         (mostly translation, see implementation details)
-# Licence: BSD 3 clause
+# License: BSD 3 clause
 
 from __future__ import print_function
 
@@ -11,13 +11,17 @@ from scipy import linalg, optimize
 
 from ..base import BaseEstimator, RegressorMixin
 from ..metrics.pairwise import manhattan_distances
-from ..utils import array2d, check_random_state, check_arrays
+from ..utils import check_random_state, check_array, check_X_y
+from ..utils.validation import check_is_fitted
 from . import regression_models as regression
 from . import correlation_models as correlation
+from ..utils import deprecated
 
 MACHINE_EPSILON = np.finfo(np.double).eps
 
 
+@deprecated("l1_cross_distances was deprecated in version 0.18 "
+            "and will be removed in 0.20.")
 def l1_cross_distances(X):
     """
     Computes the nonzero componentwise L1 cross-distances between the vectors
@@ -39,7 +43,7 @@ def l1_cross_distances(X):
         The indices i and j of the vectors in X associated to the cross-
         distances in D: D[k] = np.abs(X[ij[k, 0]] - Y[ij[k, 1]]).
     """
-    X = array2d(X)
+    X = check_array(X)
     n_samples, n_features = X.shape
     n_nonzero_cross_dist = n_samples * (n_samples - 1) // 2
     ij = np.zeros((n_nonzero_cross_dist, 2), dtype=np.int)
@@ -55,8 +59,15 @@ def l1_cross_distances(X):
     return D, ij
 
 
+@deprecated("GaussianProcess was deprecated in version 0.18 and will be "
+            "removed in 0.20. Use the GaussianProcessRegressor instead.")
 class GaussianProcess(BaseEstimator, RegressorMixin):
-    """The Gaussian Process model class.
+    """The legacy Gaussian Process model class.
+
+    Note that this class was deprecated in version 0.18 and will be
+    removed in 0.20. Use the GaussianProcessRegressor instead.
+
+    Read more in the :ref:`User Guide <gaussian_process>`.
 
     Parameters
     ----------
@@ -165,11 +176,11 @@ class GaussianProcess(BaseEstimator, RegressorMixin):
 
     Attributes
     ----------
-    `theta_`: array
+    theta_ : array
         Specified theta OR the best set of autocorrelation parameters (the \
         sought maximizer of the reduced likelihood function).
 
-    `reduced_likelihood_function_value_`: array
+    reduced_likelihood_function_value_ : array
         The optimal reduced likelihood function value.
 
     Examples
@@ -193,12 +204,12 @@ class GaussianProcess(BaseEstimator, RegressorMixin):
 
     .. [NLNS2002] `H.B. Nielsen, S.N. Lophaven, H. B. Nielsen and J.
         Sondergaard.  DACE - A MATLAB Kriging Toolbox.` (2002)
-        http://www2.imm.dtu.dk/~hbn/dace/dace.pdf
+        http://imedea.uib-csic.es/master/cambioglobal/Modulo_V_cod101615/Lab/lab_maps/krigging/DACE-krigingsoft/dace/dace.pdf
 
     .. [WBSWM1992] `W.J. Welch, R.J. Buck, J. Sacks, H.P. Wynn, T.J. Mitchell,
         and M.D.  Morris (1992). Screening, predicting, and computer
         experiments.  Technometrics, 34(1) 15--25.`
-        http://www.jstor.org/pss/1269548
+        http://www.jstor.org/stable/1269548
     """
 
     _regression_types = {
@@ -263,12 +274,10 @@ class GaussianProcess(BaseEstimator, RegressorMixin):
         self.random_state = check_random_state(self.random_state)
 
         # Force data to 2D numpy.array
-        X = array2d(X)
-        y = np.asarray(y)
+        X, y = check_X_y(X, y, multi_output=True, y_numeric=True)
         self.y_ndim_ = y.ndim
         if y.ndim == 1:
             y = y[:, np.newaxis]
-        X, y = check_arrays(X, y)
 
         # Check shapes of DOE & observations
         n_samples, n_features = X.shape
@@ -409,9 +418,10 @@ class GaussianProcess(BaseEstimator, RegressorMixin):
             An array with shape (n_eval, ) or (n_eval, n_targets) as with y,
             with the Mean Squared Error at x.
         """
+        check_is_fitted(self, "X")
 
         # Check input shapes
-        X = array2d(X)
+        X = check_array(X)
         n_eval, _ = X.shape
         n_samples, n_features = self.X.shape
         n_samples_y, n_targets = self.y.shape
@@ -472,7 +482,8 @@ class GaussianProcess(BaseEstimator, RegressorMixin):
                 if self.beta0 is None:
                     # Universal Kriging
                     u = linalg.solve_triangular(self.G.T,
-                                                np.dot(self.Ft.T, rt) - f.T)
+                                                np.dot(self.Ft.T, rt) - f.T,
+                                                lower=True)
                 else:
                     # Ordinary Kriging
                     u = np.zeros((n_targets, n_eval))
@@ -568,6 +579,7 @@ class GaussianProcess(BaseEstimator, RegressorMixin):
                 G
                         QR decomposition of the matrix Ft.
         """
+        check_is_fitted(self, "X")
 
         if theta is None:
             # Use built-in autocorrelation parameters
@@ -613,7 +625,6 @@ class GaussianProcess(BaseEstimator, RegressorMixin):
             # 0.7. The economy transform will then be available through the
             # mode='economic' argument.
             Q, G = linalg.qr(Ft, mode='economic')
-            pass
 
         sv = linalg.svd(G, compute_uv=False)
         rcondG = sv[-1] / sv[0]
@@ -714,17 +725,16 @@ class GaussianProcess(BaseEstimator, RegressorMixin):
                 else:
                     # Generate a random starting point log10-uniformly
                     # distributed between bounds
-                    log10theta0 = np.log10(self.thetaL) \
-                        + self.random_state.rand(self.theta0.size).reshape(
-                            self.theta0.shape) * np.log10(self.thetaU
-                                                          / self.thetaL)
+                    log10theta0 = (np.log10(self.thetaL)
+                                   + self.random_state.rand(*self.theta0.shape)
+                                   * np.log10(self.thetaU / self.thetaL))
                     theta0 = 10. ** log10theta0
 
                 # Run Cobyla
                 try:
                     log10_optimal_theta = \
                         optimize.fmin_cobyla(minus_reduced_likelihood_function,
-                                             np.log10(theta0), constraints,
+                                             np.log10(theta0).ravel(), constraints,
                                              iprint=0)
                 except ValueError as ve:
                     print("Optimization failed. Try increasing the ``nugget``")
@@ -755,7 +765,7 @@ class GaussianProcess(BaseEstimator, RegressorMixin):
 
         elif self.optimizer == 'Welch':
 
-            # Backup of the given atrributes
+            # Backup of the given attributes
             theta0, thetaL, thetaU = self.theta0, self.thetaL, self.thetaU
             corr = self.corr
             verbose = self.verbose
@@ -767,9 +777,9 @@ class GaussianProcess(BaseEstimator, RegressorMixin):
             # Initialize under isotropy assumption
             if verbose:
                 print("Initialize under isotropy assumption...")
-            self.theta0 = array2d(self.theta0.min())
-            self.thetaL = array2d(self.thetaL.min())
-            self.thetaU = array2d(self.thetaU.max())
+            self.theta0 = check_array(self.theta0.min())
+            self.thetaL = check_array(self.thetaL.min())
+            self.thetaU = check_array(self.thetaU.max())
             theta_iso, optimal_rlf_value_iso, par_iso = \
                 self._arg_max_reduced_likelihood_function()
             optimal_theta = theta_iso + np.zeros(theta0.shape)
@@ -780,21 +790,22 @@ class GaussianProcess(BaseEstimator, RegressorMixin):
             for i in self.random_state.permutation(theta0.size):
                 if verbose:
                     print("Proceeding along dimension %d..." % (i + 1))
-                self.theta0 = array2d(theta_iso)
-                self.thetaL = array2d(thetaL[0, i])
-                self.thetaU = array2d(thetaU[0, i])
+                self.theta0 = check_array(theta_iso)
+                self.thetaL = check_array(thetaL[0, i])
+                self.thetaU = check_array(thetaU[0, i])
 
                 def corr_cut(t, d):
-                    return corr(array2d(np.hstack([optimal_theta[0][0:i],
-                                                   t[0],
-                                                   optimal_theta[0][(i + 1)::]]
-                                                  )), d)
+                    return corr(check_array(np.hstack([optimal_theta[0][0:i],
+                                                       t[0],
+                                                       optimal_theta[0][(i +
+                                                                         1)::]])),
+                                d)
 
                 self.corr = corr_cut
                 optimal_theta[0, i], optimal_rlf_value, optimal_par = \
                     self._arg_max_reduced_likelihood_function()
 
-            # Restore the given atrributes
+            # Restore the given attributes
             self.theta0, self.thetaL, self.thetaU = theta0, thetaL, thetaU
             self.corr = corr
             self.optimizer = 'Welch'
@@ -821,7 +832,7 @@ class GaussianProcess(BaseEstimator, RegressorMixin):
 
         # Check regression weights if given (Ordinary Kriging)
         if self.beta0 is not None:
-            self.beta0 = array2d(self.beta0)
+            self.beta0 = np.atleast_2d(self.beta0)
             if self.beta0.shape[1] != 1:
                 # Force to column vector
                 self.beta0 = self.beta0.T
@@ -841,12 +852,12 @@ class GaussianProcess(BaseEstimator, RegressorMixin):
                              "'light', %s was given." % self.storage_mode)
 
         # Check correlation parameters
-        self.theta0 = array2d(self.theta0)
+        self.theta0 = np.atleast_2d(self.theta0)
         lth = self.theta0.size
 
         if self.thetaL is not None and self.thetaU is not None:
-            self.thetaL = array2d(self.thetaL)
-            self.thetaU = array2d(self.thetaU)
+            self.thetaL = np.atleast_2d(self.thetaL)
+            self.thetaU = np.atleast_2d(self.thetaU)
             if self.thetaL.size != lth or self.thetaU.size != lth:
                 raise ValueError("theta0, thetaL and thetaU must have the "
                                  "same length.")
@@ -878,7 +889,7 @@ class GaussianProcess(BaseEstimator, RegressorMixin):
                              "or array of length n_samples.")
 
         # Check optimizer
-        if not self.optimizer in self._optimizer_types:
+        if self.optimizer not in self._optimizer_types:
             raise ValueError("optimizer should be one of %s"
                              % self._optimizer_types)
 
