@@ -5,13 +5,14 @@
 # License: BSD 3 clause
 
 import numpy as np
+import warnings
 
 from ..base import BaseEstimator, MetaEstimatorMixin, RegressorMixin, clone
 from ..utils import check_random_state, check_array, check_consistent_length
 from ..utils.random import sample_without_replacement
 from ..utils.validation import check_is_fitted
 from .base import LinearRegression
-
+from ..utils.validation import has_fit_parameter
 
 _EPSILON = np.spacing(1)
 
@@ -134,6 +135,22 @@ class RANSACRegressor(BaseEstimator, MetaEstimatorMixin, RegressorMixin):
 
             lambda dy: np.sum(np.abs(dy), axis=1)
 
+        NOTE: residual_metric is deprecated from 0.18 and will be removed in 0.20
+        Use ``loss`` instead.
+
+    loss: string, callable, optional, default "absolute_loss"
+        String inputs, "absolute_loss" and "squared_loss" are supported which
+        find the absolute loss and squared loss per sample
+        respectively.
+
+        If ``loss`` is a callable, then it should be a function that takes
+        two arrays as inputs, the true and predicted value and returns a 1-D
+        array with the ``i``th value of the array corresponding to the loss
+        on `X[i]`.
+
+        If the loss on a sample is greater than the ``residual_threshold``, then
+        this sample is classified as an outlier.
+
     random_state : integer or numpy.RandomState, optional
         The generator used to initialize the centers. If an integer is
         given, it fixes the seed. Defaults to the global numpy random
@@ -153,7 +170,7 @@ class RANSACRegressor(BaseEstimator, MetaEstimatorMixin, RegressorMixin):
 
     References
     ----------
-    .. [1] http://en.wikipedia.org/wiki/RANSAC
+    .. [1] https://en.wikipedia.org/wiki/RANSAC
     .. [2] http://www.cs.columbia.edu/~belhumeur/courses/compPhoto/ransac.pdf
     .. [3] http://www.bmva.org/bmvc/2009/Papers/Paper355/Paper355.pdf
     """
@@ -163,7 +180,7 @@ class RANSACRegressor(BaseEstimator, MetaEstimatorMixin, RegressorMixin):
                  is_model_valid=None, max_trials=100,
                  stop_n_inliers=np.inf, stop_score=np.inf,
                  stop_probability=0.99, residual_metric=None,
-                 random_state=None):
+                 loss='absolute_loss', random_state=None):
 
         self.base_estimator = base_estimator
         self.min_samples = min_samples
@@ -176,8 +193,9 @@ class RANSACRegressor(BaseEstimator, MetaEstimatorMixin, RegressorMixin):
         self.stop_probability = stop_probability
         self.residual_metric = residual_metric
         self.random_state = random_state
+        self.loss = loss
 
-    def fit(self, X, y):
+    def fit(self, X, y, sample_weight=None):
         """Fit estimator using RANSAC algorithm.
 
         Parameters
@@ -188,6 +206,11 @@ class RANSACRegressor(BaseEstimator, MetaEstimatorMixin, RegressorMixin):
         y : array-like, shape = [n_samples] or [n_samples, n_targets]
             Target values.
 
+        sample_weight: array-like, shape = [n_samples]
+            Individual weights for each sample
+            raises error if sample_weight is passed and base_estimator
+            fit method does not support it.
+
         Raises
         ------
         ValueError
@@ -231,10 +254,34 @@ class RANSACRegressor(BaseEstimator, MetaEstimatorMixin, RegressorMixin):
         else:
             residual_threshold = self.residual_threshold
 
-        if self.residual_metric is None:
-            residual_metric = lambda dy: np.sum(np.abs(dy), axis=1)
+        if self.residual_metric is not None:
+            warnings.warn(
+                "'residual_metric' was deprecated in version 0.18 and "
+                "will be removed in version 0.20. Use 'loss' instead.",
+                DeprecationWarning)
+
+        if self.loss == "absolute_loss":
+            if y.ndim == 1:
+                loss_function = lambda y_true, y_pred: np.abs(y_true - y_pred)
+            else:
+                loss_function = lambda \
+                    y_true, y_pred: np.sum(np.abs(y_true - y_pred), axis=1)
+
+        elif self.loss == "squared_loss":
+            if y.ndim == 1:
+                loss_function = lambda y_true, y_pred: (y_true - y_pred) ** 2
+            else:
+                loss_function = lambda \
+                    y_true, y_pred: np.sum((y_true - y_pred) ** 2, axis=1)
+
+        elif callable(self.loss):
+            loss_function = self.loss
+
         else:
-            residual_metric = self.residual_metric
+            raise ValueError(
+                "loss should be 'absolute_loss', 'squared_loss' or a callable."
+                "Got %s. " % self.loss)
+
 
         random_state = check_random_state(self.random_state)
 
@@ -243,6 +290,17 @@ class RANSACRegressor(BaseEstimator, MetaEstimatorMixin, RegressorMixin):
         except ValueError:
             pass
 
+        estimator_fit_has_sample_weight = has_fit_parameter(base_estimator,
+                                                            "sample_weight")
+        estimator_name = type(base_estimator).__name__
+        if (sample_weight is not None and not
+                estimator_fit_has_sample_weight):
+            raise ValueError("%s does not support sample_weight. Samples"
+                             " weights are only used for the calibration"
+                             " itself." % estimator_name)
+        if sample_weight is not None:
+            sample_weight = np.asarray(sample_weight)
+
         n_inliers_best = 0
         score_best = np.inf
         inlier_mask_best = None
@@ -269,7 +327,11 @@ class RANSACRegressor(BaseEstimator, MetaEstimatorMixin, RegressorMixin):
                 continue
 
             # fit model for current random sample set
-            base_estimator.fit(X_subset, y_subset)
+            if sample_weight is None:
+                base_estimator.fit(X_subset, y_subset)
+            else:
+                base_estimator.fit(X_subset, y_subset,
+                                   sample_weight=sample_weight[subset_idxs])
 
             # check if estimated model is valid
             if (self.is_model_valid is not None and not
@@ -278,10 +340,15 @@ class RANSACRegressor(BaseEstimator, MetaEstimatorMixin, RegressorMixin):
 
             # residuals of all data for current random sample model
             y_pred = base_estimator.predict(X)
-            diff = y_pred - y
-            if diff.ndim == 1:
-                diff = diff.reshape(-1, 1)
-            residuals_subset = residual_metric(diff)
+
+            # XXX: Deprecation: Remove this if block in 0.20
+            if self.residual_metric is not None:
+                diff = y_pred - y
+                if diff.ndim == 1:
+                    diff = diff.reshape(-1, 1)
+                residuals_subset = self.residual_metric(diff)
+            else:
+                residuals_subset = loss_function(y, y_pred)
 
             # classify data into inliers and outliers
             inlier_mask_subset = residuals_subset < residual_threshold
