@@ -62,7 +62,7 @@ def barycenter_weights(X, Z, reg=1e-3):
     return B
 
 
-def barycenter_kneighbors_graph(X, n_neighbors, reg=1e-3):
+def barycenter_kneighbors_graph(X, n_neighbors, reg=1e-3, n_jobs=1):
     """Computes the barycenter weighted graph of k-Neighbors for points in X
 
     Parameters
@@ -80,6 +80,10 @@ def barycenter_kneighbors_graph(X, n_neighbors, reg=1e-3):
         problem. Only relevant if mode='barycenter'. If None, use the
         default.
 
+    n_jobs : int, optional (default = 1)
+        The number of parallel jobs to run for neighbors search.
+        If ``-1``, then the number of jobs is set to the number of CPU cores.
+
     Returns
     -------
     A : sparse matrix in CSR format, shape = [n_samples, n_samples]
@@ -90,7 +94,7 @@ def barycenter_kneighbors_graph(X, n_neighbors, reg=1e-3):
     sklearn.neighbors.kneighbors_graph
     sklearn.neighbors.radius_neighbors_graph
     """
-    knn = NearestNeighbors(n_neighbors + 1).fit(X)
+    knn = NearestNeighbors(n_neighbors + 1, n_jobs=n_jobs).fit(X)
     X = knn._fit_X
     n_samples = X.shape[0]
     ind = knn.kneighbors(X, return_distance=False)[:, 1:]
@@ -148,7 +152,8 @@ def null_space(M, k, k_skip=1, eigen_solver='arpack', tol=1E-6, max_iter=100,
 
     if eigen_solver == 'arpack':
         random_state = check_random_state(random_state)
-        v0 = random_state.rand(M.shape[0])
+        # initialize with [-1,1] as in ARPACK
+        v0 = random_state.uniform(-1, 1, M.shape[0])
         try:
             eigen_values, eigen_vectors = eigsh(M, k + k_skip, sigma=0.0,
                                                 tol=tol, maxiter=max_iter,
@@ -177,7 +182,7 @@ def null_space(M, k, k_skip=1, eigen_solver='arpack', tol=1E-6, max_iter=100,
 def locally_linear_embedding(
         X, n_neighbors, n_components, reg=1e-3, eigen_solver='auto', tol=1e-6,
         max_iter=100, method='standard', hessian_tol=1E-4, modified_tol=1E-12,
-        random_state=None):
+        random_state=None, n_jobs=1):
     """Perform a Locally Linear Embedding analysis on the data.
 
     Read more in the :ref:`User Guide <locally_linear_embedding>`.
@@ -243,6 +248,10 @@ def locally_linear_embedding(
         The generator or seed used to determine the starting vector for arpack
         iterations.  Defaults to numpy.random.
 
+    n_jobs : int, optional (default = 1)
+        The number of parallel jobs to run for neighbors search.
+        If ``-1``, then the number of jobs is set to the number of CPU cores.
+
     Returns
     -------
     Y : array-like, shape [n_samples, n_components]
@@ -273,7 +282,7 @@ def locally_linear_embedding(
     if method not in ('standard', 'hessian', 'modified', 'ltsa'):
         raise ValueError("unrecognized method '%s'" % method)
 
-    nbrs = NearestNeighbors(n_neighbors=n_neighbors + 1)
+    nbrs = NearestNeighbors(n_neighbors=n_neighbors + 1, n_jobs=n_jobs)
     nbrs.fit(X)
     X = nbrs._fit_X
 
@@ -292,7 +301,7 @@ def locally_linear_embedding(
 
     if method == 'standard':
         W = barycenter_kneighbors_graph(
-            nbrs, n_neighbors=n_neighbors, reg=reg)
+            nbrs, n_neighbors=n_neighbors, reg=reg, n_jobs=n_jobs)
 
         # we'll compute M = (I-W)'(I-W)
         # depending on the solver, we'll do this differently
@@ -315,10 +324,10 @@ def locally_linear_embedding(
                                     return_distance=False)
         neighbors = neighbors[:, 1:]
 
-        Yi = np.empty((n_neighbors, 1 + n_components + dp), dtype=np.float)
+        Yi = np.empty((n_neighbors, 1 + n_components + dp), dtype=np.float64)
         Yi[:, 0] = 1
 
-        M = np.zeros((N, N), dtype=np.float)
+        M = np.zeros((N, N), dtype=np.float64)
 
         use_svd = (n_neighbors > d_in)
 
@@ -326,7 +335,7 @@ def locally_linear_embedding(
             Gi = X[neighbors[i]]
             Gi -= Gi.mean(0)
 
-            #build Hessian estimator
+            # build Hessian estimator
             if use_svd:
                 U = svd(Gi, full_matrices=0)[0]
             else:
@@ -337,8 +346,8 @@ def locally_linear_embedding(
 
             j = 1 + n_components
             for k in range(n_components):
-                Yi[:, j:j + n_components - k] = (U[:, k:k + 1]
-                                                 * U[:, k:n_components])
+                Yi[:, j:j + n_components - k] = (U[:, k:k + 1] *
+                                                 U[:, k:n_components])
                 j += n_components - k
 
             Q, R = qr(Yi)
@@ -364,14 +373,14 @@ def locally_linear_embedding(
                                     return_distance=False)
         neighbors = neighbors[:, 1:]
 
-        #find the eigenvectors and eigenvalues of each local covariance
+        # find the eigenvectors and eigenvalues of each local covariance
         # matrix. We want V[i] to be a [n_neighbors x n_neighbors] matrix,
         # where the columns are eigenvectors
         V = np.zeros((N, n_neighbors, n_neighbors))
         nev = min(d_in, n_neighbors)
         evals = np.zeros([N, nev])
 
-        #choose the most efficient way to find the eigenvectors
+        # choose the most efficient way to find the eigenvectors
         use_svd = (n_neighbors > d_in)
 
         if use_svd:
@@ -388,7 +397,7 @@ def locally_linear_embedding(
                 evals[i] = evi[::-1]
                 V[i] = vi[:, ::-1]
 
-        #find regularized weights: this is like normal LLE.
+        # find regularized weights: this is like normal LLE.
         # because we've already computed the SVD of each covariance matrix,
         # it's faster to use this rather than np.linalg.solve
         reg = 1E-3 * evals.sum(1)
@@ -402,12 +411,12 @@ def locally_linear_embedding(
             w_reg[i] = np.dot(V[i], tmp[i])
         w_reg /= w_reg.sum(1)[:, None]
 
-        #calculate eta: the median of the ratio of small to large eigenvalues
+        # calculate eta: the median of the ratio of small to large eigenvalues
         # across the points.  This is used to determine s_i, below
         rho = evals[:, n_components:].sum(1) / evals[:, :n_components].sum(1)
         eta = np.median(rho)
 
-        #find s_i, the size of the "almost null space" for each point:
+        # find s_i, the size of the "almost null space" for each point:
         # this is the size of the largest set of eigenvalues
         # such that Sum[v; v in set]/Sum[v; v not in set] < eta
         s_range = np.zeros(N, dtype=int)
@@ -417,17 +426,17 @@ def locally_linear_embedding(
             s_range[i] = np.searchsorted(eta_range[i, ::-1], eta)
         s_range += n_neighbors - nev  # number of zero eigenvalues
 
-        #Now calculate M.
+        # Now calculate M.
         # This is the [N x N] matrix whose null space is the desired embedding
-        M = np.zeros((N, N), dtype=np.float)
+        M = np.zeros((N, N), dtype=np.float64)
         for i in range(N):
             s_i = s_range[i]
 
-            #select bottom s_i eigenvectors and calculate alpha
+            # select bottom s_i eigenvectors and calculate alpha
             Vi = V[i, :, n_neighbors - s_i:]
             alpha_i = np.linalg.norm(Vi.sum(0)) / np.sqrt(s_i)
 
-            #compute Householder matrix which satisfies
+            # compute Householder matrix which satisfies
             #  Hi*Vi.T*ones(n_neighbors) = alpha_i*ones(s)
             # using prescription from paper
             h = alpha_i * np.ones(s_i) - np.dot(Vi.T, np.ones(n_neighbors))
@@ -438,20 +447,20 @@ def locally_linear_embedding(
             else:
                 h /= norm_h
 
-            #Householder matrix is
+            # Householder matrix is
             #  >> Hi = np.identity(s_i) - 2*np.outer(h,h)
-            #Then the weight matrix is
+            # Then the weight matrix is
             #  >> Wi = np.dot(Vi,Hi) + (1-alpha_i) * w_reg[i,:,None]
-            #We do this much more efficiently:
-            Wi = (Vi - 2 * np.outer(np.dot(Vi, h), h)
-                  + (1 - alpha_i) * w_reg[i, :, None])
+            # We do this much more efficiently:
+            Wi = (Vi - 2 * np.outer(np.dot(Vi, h), h) +
+                  (1 - alpha_i) * w_reg[i, :, None])
 
-            #Update M as follows:
+            # Update M as follows:
             # >> W_hat = np.zeros( (N,s_i) )
             # >> W_hat[neighbors[i],:] = Wi
             # >> W_hat[i] -= 1
             # >> M += np.dot(W_hat,W_hat.T)
-            #We can do this much more efficiently:
+            # We can do this much more efficiently:
             nbrs_x, nbrs_y = np.meshgrid(neighbors[i], neighbors[i])
             M[nbrs_x, nbrs_y] += np.dot(Wi, Wi.T)
             Wi_sum1 = Wi.sum(1)
@@ -562,6 +571,10 @@ class LocallyLinearEmbedding(BaseEstimator, TransformerMixin):
         The generator or seed used to determine the starting vector for arpack
         iterations.  Defaults to numpy.random.
 
+    n_jobs : int, optional (default = 1)
+        The number of parallel jobs to run.
+        If ``-1``, then the number of jobs is set to the number of CPU cores.
+
     Attributes
     ----------
     embedding_vectors_ : array-like, shape [n_components, n_samples]
@@ -593,8 +606,7 @@ class LocallyLinearEmbedding(BaseEstimator, TransformerMixin):
     def __init__(self, n_neighbors=5, n_components=2, reg=1E-3,
                  eigen_solver='auto', tol=1E-6, max_iter=100,
                  method='standard', hessian_tol=1E-4, modified_tol=1E-12,
-                 neighbors_algorithm='auto', random_state=None):
-
+                 neighbors_algorithm='auto', random_state=None, n_jobs=1):
         self.n_neighbors = n_neighbors
         self.n_components = n_components
         self.reg = reg
@@ -606,10 +618,12 @@ class LocallyLinearEmbedding(BaseEstimator, TransformerMixin):
         self.modified_tol = modified_tol
         self.random_state = random_state
         self.neighbors_algorithm = neighbors_algorithm
+        self.n_jobs = n_jobs
 
     def _fit_transform(self, X):
         self.nbrs_ = NearestNeighbors(self.n_neighbors,
-                                      algorithm=self.neighbors_algorithm)
+                                      algorithm=self.neighbors_algorithm,
+                                      n_jobs=self.n_jobs)
 
         random_state = check_random_state(self.random_state)
         X = check_array(X)
@@ -620,7 +634,7 @@ class LocallyLinearEmbedding(BaseEstimator, TransformerMixin):
                 eigen_solver=self.eigen_solver, tol=self.tol,
                 max_iter=self.max_iter, method=self.method,
                 hessian_tol=self.hessian_tol, modified_tol=self.modified_tol,
-                random_state=random_state, reg=self.reg)
+                random_state=random_state, reg=self.reg, n_jobs=self.n_jobs)
 
     def fit(self, X, y=None):
         """Compute the embedding vectors for data X
