@@ -1,52 +1,25 @@
 # -*- coding: utf-8 -*-
-"""
-The :mod:`sklearn.metrics.pairwise` submodule implements utilities to evaluate
-pairwise distances, paired distances or affinity of sets of samples.
-
-This module contains both distance metrics and kernels. A brief summary is
-given on the two here.
-
-Distance metrics are a function d(a, b) such that d(a, b) < d(a, c) if objects
-a and b are considered "more similar" to objects a and c. Two objects exactly
-alike would have a distance of zero.
-One of the most popular examples is Euclidean distance.
-To be a 'true' metric, it must obey the following four conditions::
-
-    1. d(a, b) >= 0, for all a and b
-    2. d(a, b) == 0, if and only if a = b, positive definiteness
-    3. d(a, b) == d(b, a), symmetry
-    4. d(a, c) <= d(a, b) + d(b, c), the triangle inequality
-
-Kernels are measures of similarity, i.e. ``s(a, b) > s(a, c)``
-if objects ``a`` and ``b`` are considered "more similar" to objects
-``a`` and ``c``. A kernel must also be positive semi-definite.
-
-There are a number of ways to convert between a distance metric and a
-similarity measure, such as a kernel. Let D be the distance, and S be the
-kernel:
-
-    1. ``S = np.exp(-D * gamma)``, where one heuristic for choosing
-       ``gamma`` is ``1 / num_features``
-    2. ``S = 1. / (D / np.max(D))``
-"""
 
 # Authors: Alexandre Gramfort <alexandre.gramfort@inria.fr>
 #          Mathieu Blondel <mathieu@mblondel.org>
 #          Robert Layton <robertlayton@gmail.com>
 #          Andreas Mueller <amueller@ais.uni-bonn.de>
 #          Philippe Gervais <philippe.gervais@inria.fr>
-#          Lars Buitinck <larsmans@gmail.com>
+#          Lars Buitinck
+#          Joel Nothman <joel.nothman@gmail.com>
 # License: BSD 3 clause
 
+import itertools
+
 import numpy as np
 from scipy.spatial import distance
 from scipy.sparse import csr_matrix
 from scipy.sparse import issparse
 
-from ..utils import array2d, atleast2d_or_csr
+from ..utils import check_array
 from ..utils import gen_even_slices
 from ..utils import gen_batches
-from ..utils import safe_asarray
+from ..utils.fixes import partial
 from ..utils.extmath import row_norms, safe_sparse_dot
 from ..preprocessing import normalize
 from ..externals.joblib import Parallel
@@ -57,7 +30,31 @@ from .pairwise_fast import _chi2_kernel_fast, _sparse_manhattan
 
 
 # Utility Functions
-def check_pairwise_arrays(X, Y):
+def _return_float_dtype(X, Y):
+    """
+    1. If dtype of X and Y is float32, then dtype float32 is returned.
+    2. Else dtype float is returned.
+    """
+    if not issparse(X) and not isinstance(X, np.ndarray):
+        X = np.asarray(X)
+
+    if Y is None:
+        Y_dtype = X.dtype
+    elif not issparse(Y) and not isinstance(Y, np.ndarray):
+        Y = np.asarray(Y)
+        Y_dtype = Y.dtype
+    else:
+        Y_dtype = Y.dtype
+
+    if X.dtype == Y_dtype == np.float32:
+        dtype = np.float32
+    else:
+        dtype = np.float
+
+    return X, Y, dtype
+
+
+def check_pairwise_arrays(X, Y, precomputed=False, dtype=None):
     """ Set X and Y appropriately and checks inputs
 
     If Y is None, it is set as a pointer to X (i.e. not a copy).
@@ -67,8 +64,9 @@ def check_pairwise_arrays(X, Y):
 
     Specifically, this function first ensures that both X and Y are arrays,
     then checks that they are at least two dimensional while ensuring that
-    their elements are floats. Finally, the function checks that the size
-    of the second dimension of the two arrays is equal.
+    their elements are floats (or dtype if provided). Finally, the function
+    checks that the size of the second dimension of the two arrays is equal, or
+    the equivalent check for a precomputed distance matrix.
 
     Parameters
     ----------
@@ -76,6 +74,16 @@ def check_pairwise_arrays(X, Y):
 
     Y : {array-like, sparse matrix}, shape (n_samples_b, n_features)
 
+    precomputed : bool
+        True if X is to be treated as precomputed distances to the samples in
+        Y.
+
+    dtype : string, type, list of types or None (default=None)
+        Data type required for X and Y. If None, the dtype will be an
+        appropriate float type selected by _return_float_dtype.
+
+        .. versionadded:: 0.18
+
     Returns
     -------
     safe_X : {array-like, sparse matrix}, shape (n_samples_a, n_features)
@@ -86,22 +94,33 @@ def check_pairwise_arrays(X, Y):
         If Y was None, safe_Y will be a pointer to X.
 
     """
+    X, Y, dtype_float = _return_float_dtype(X, Y)
+
+    warn_on_dtype = dtype is not None
+    estimator = 'check_pairwise_arrays'
+    if dtype is None:
+        dtype = dtype_float
+
     if Y is X or Y is None:
-        X = Y = atleast2d_or_csr(X)
+        X = Y = check_array(X, accept_sparse='csr', dtype=dtype,
+                            warn_on_dtype=warn_on_dtype, estimator=estimator)
     else:
-        X = atleast2d_or_csr(X)
-        Y = atleast2d_or_csr(Y)
-    if X.shape[1] != Y.shape[1]:
+        X = check_array(X, accept_sparse='csr', dtype=dtype,
+                        warn_on_dtype=warn_on_dtype, estimator=estimator)
+        Y = check_array(Y, accept_sparse='csr', dtype=dtype,
+                        warn_on_dtype=warn_on_dtype, estimator=estimator)
+
+    if precomputed:
+        if X.shape[1] != Y.shape[0]:
+            raise ValueError("Precomputed metric requires shape "
+                             "(n_queries, n_indexed). Got (%d, %d) "
+                             "for %d indexed." %
+                             (X.shape[0], X.shape[1], Y.shape[0]))
+    elif X.shape[1] != Y.shape[1]:
         raise ValueError("Incompatible dimension for X and Y matrices: "
                          "X.shape[1] == %d while Y.shape[1] == %d" % (
                              X.shape[1], Y.shape[1]))
 
-    if not (X.dtype == Y.dtype == np.float32):
-        if Y is X:
-            X = Y = safe_asarray(X, dtype=np.float)
-        else:
-            X = safe_asarray(X, dtype=np.float)
-            Y = safe_asarray(Y, dtype=np.float)
     return X, Y
 
 
@@ -140,7 +159,8 @@ def check_paired_arrays(X, Y):
 
 
 # Pairwise distances
-def euclidean_distances(X, Y=None, Y_norm_squared=None, squared=False):
+def euclidean_distances(X, Y=None, Y_norm_squared=None, squared=False,
+                        X_norm_squared=None):
     """
     Considering the rows of X (and Y=X) as vectors, compute the
     distance matrix between each pair of vectors.
@@ -152,13 +172,15 @@ def euclidean_distances(X, Y=None, Y_norm_squared=None, squared=False):
 
     This formulation has two advantages over other ways of computing distances.
     First, it is computationally efficient when dealing with sparse data.
-    Second, if x varies but y remains unchanged, then the right-most dot
-    product `dot(y, y)` can be pre-computed.
+    Second, if one argument varies but the other remains unchanged, then
+    `dot(x, x)` and/or `dot(y, y)` can be pre-computed.
 
     However, this is not the most precise way of doing this computation, and
     the distance matrix returned by this function may not be exactly
     symmetric as required by, e.g., ``scipy.spatial.distance`` functions.
 
+    Read more in the :ref:`User Guide <metrics>`.
+
     Parameters
     ----------
     X : {array-like, sparse matrix}, shape (n_samples_1, n_features)
@@ -172,6 +194,10 @@ def euclidean_distances(X, Y=None, Y_norm_squared=None, squared=False):
     squared : boolean, optional
         Return squared Euclidean distances.
 
+    X_norm_squared : array-like, shape = [n_samples_1], optional
+        Pre-computed dot-products of vectors in X (e.g.,
+        ``(X**2).sum(axis=1)``)
+
     Returns
     -------
     distances : {array, sparse matrix}, shape (n_samples_1, n_samples_2)
@@ -193,24 +219,29 @@ def euclidean_distances(X, Y=None, Y_norm_squared=None, squared=False):
     --------
     paired_distances : distances betweens pairs of elements of X and Y.
     """
-    # should not need X_norm_squared because if you could precompute that as
-    # well as Y, then you should just pre-compute the output and not even
-    # call this function.
     X, Y = check_pairwise_arrays(X, Y)
 
-    if Y_norm_squared is not None:
-        YY = array2d(Y_norm_squared)
+    if X_norm_squared is not None:
+        XX = check_array(X_norm_squared)
+        if XX.shape == (1, X.shape[0]):
+            XX = XX.T
+        elif XX.shape != (X.shape[0], 1):
+            raise ValueError(
+                "Incompatible dimensions for X and X_norm_squared")
+    else:
+        XX = row_norms(X, squared=True)[:, np.newaxis]
+
+    if X is Y:  # shortcut in the common case euclidean_distances(X, X)
+        YY = XX.T
+    elif Y_norm_squared is not None:
+        YY = np.atleast_2d(Y_norm_squared)
+
         if YY.shape != (1, Y.shape[0]):
             raise ValueError(
                 "Incompatible dimensions for Y and Y_norm_squared")
     else:
         YY = row_norms(Y, squared=True)[np.newaxis, :]
 
-    if X is Y:  # shortcut in the common case euclidean_distances(X, X)
-        XX = YY.T
-    else:
-        XX = row_norms(X, squared=True)[:, np.newaxis]
-
     distances = safe_sparse_dot(X, Y.T, dense_output=True)
     distances *= -2
     distances += XX
@@ -253,7 +284,7 @@ def pairwise_distances_argmin_min(X, Y, axis=1, metric="euclidean",
         can be changed for fine-tuning. The larger the number, the larger the
         memory usage.
 
-    metric : string or callable
+    metric : string or callable, default 'euclidean'
         metric to use for distance computation. Any metric from scikit-learn
         or scipy.spatial.distance can be used.
 
@@ -272,8 +303,9 @@ def pairwise_distances_argmin_min(X, Y, axis=1, metric="euclidean",
 
         - from scipy.spatial.distance: ['braycurtis', 'canberra', 'chebyshev',
           'correlation', 'dice', 'hamming', 'jaccard', 'kulsinski',
-          'mahalanobis', 'matching', 'minkowski', 'rogerstanimoto', 'russellrao',
-          'seuclidean', 'sokalmichener', 'sokalsneath', 'sqeuclidean', 'yule']
+          'mahalanobis', 'matching', 'minkowski', 'rogerstanimoto',
+          'russellrao', 'seuclidean', 'sokalmichener', 'sokalsneath',
+          'sqeuclidean', 'yule']
 
         See the documentation for scipy.spatial.distance for details on these
         metrics.
@@ -281,6 +313,9 @@ def pairwise_distances_argmin_min(X, Y, axis=1, metric="euclidean",
     metric_kwargs : dict, optional
         Keyword arguments to pass to specified metric function.
 
+    axis : int, optional, default 1
+        Axis along which the argmin and distances are to be computed.
+
     Returns
     -------
     argmin : numpy.ndarray
@@ -340,11 +375,8 @@ def pairwise_distances_argmin_min(X, Y, axis=1, metric="euclidean",
                                  min_indices]
 
             flags = values[chunk_x] > min_values
-            indices[chunk_x] = np.where(
-                flags, min_indices + chunk_y.start, indices[chunk_x])
-
-            values[chunk_x] = np.where(
-                flags, min_values, values[chunk_x])
+            indices[chunk_x][flags] = min_indices[flags] + chunk_y.start
+            values[chunk_x][flags] = min_values[flags]
 
     if metric == "euclidean" and not metric_kwargs.get("squared", False):
         np.sqrt(values, values)
@@ -352,7 +384,7 @@ def pairwise_distances_argmin_min(X, Y, axis=1, metric="euclidean",
 
 
 def pairwise_distances_argmin(X, Y, axis=1, metric="euclidean",
-                              batch_size=500, metric_kwargs={}):
+                              batch_size=500, metric_kwargs=None):
     """Compute minimum distances between one point and a set of points.
 
     This function computes for each row in X, the index of the row of Y which
@@ -367,8 +399,12 @@ def pairwise_distances_argmin(X, Y, axis=1, metric="euclidean",
     This function works with dense 2D arrays only.
 
     Parameters
-    ==========
-    X, Y : array-like
+    ----------
+    X : array-like
+        Arrays containing points. Respective shapes (n_samples1, n_features)
+        and (n_samples2, n_features)
+
+    Y : array-like
         Arrays containing points. Respective shapes (n_samples1, n_features)
         and (n_samples2, n_features)
 
@@ -398,8 +434,9 @@ def pairwise_distances_argmin(X, Y, axis=1, metric="euclidean",
 
         - from scipy.spatial.distance: ['braycurtis', 'canberra', 'chebyshev',
           'correlation', 'dice', 'hamming', 'jaccard', 'kulsinski',
-          'mahalanobis', 'matching', 'minkowski', 'rogerstanimoto', 'russellrao',
-          'seuclidean', 'sokalmichener', 'sokalsneath', 'sqeuclidean', 'yule']
+          'mahalanobis', 'matching', 'minkowski', 'rogerstanimoto',
+          'russellrao', 'seuclidean', 'sokalmichener', 'sokalsneath',
+          'sqeuclidean', 'yule']
 
         See the documentation for scipy.spatial.distance for details on these
         metrics.
@@ -407,16 +444,22 @@ def pairwise_distances_argmin(X, Y, axis=1, metric="euclidean",
     metric_kwargs : dict
         keyword arguments to pass to specified metric function.
 
+    axis : int, optional, default 1
+        Axis along which the argmin and distances are to be computed.
+
     Returns
-    =======
+    -------
     argmin : numpy.ndarray
         Y[argmin[i], :] is the row in Y that is closest to X[i, :].
 
     See also
-    ========
+    --------
     sklearn.metrics.pairwise_distances
     sklearn.metrics.pairwise_distances_argmin_min
     """
+    if metric_kwargs is None:
+        metric_kwargs = {}
+
     return pairwise_distances_argmin_min(X, Y, axis, metric, batch_size,
                                          metric_kwargs)[0]
 
@@ -428,6 +471,8 @@ def manhattan_distances(X, Y=None, sum_over_features=True,
     With sum_over_features equal to False it returns the componentwise
     distances.
 
+    Read more in the :ref:`User Guide <metrics>`.
+
     Parameters
     ----------
     X : array_like
@@ -442,9 +487,7 @@ def manhattan_distances(X, Y=None, sum_over_features=True,
         Not supported for sparse matrix inputs.
 
     size_threshold : int, default=5e8
-        Avoid creating temporary matrices bigger than size_threshold (in
-        bytes). If the problem size gets too big, the implementation then
-        breaks it down in smaller problems.
+        Unused parameter.
 
     Returns
     -------
@@ -458,11 +501,11 @@ def manhattan_distances(X, Y=None, sum_over_features=True,
     Examples
     --------
     >>> from sklearn.metrics.pairwise import manhattan_distances
-    >>> manhattan_distances(3, 3)#doctest:+ELLIPSIS
+    >>> manhattan_distances([[3]], [[3]])#doctest:+ELLIPSIS
     array([[ 0.]])
-    >>> manhattan_distances(3, 2)#doctest:+ELLIPSIS
+    >>> manhattan_distances([[3]], [[2]])#doctest:+ELLIPSIS
     array([[ 1.]])
-    >>> manhattan_distances(2, 3)#doctest:+ELLIPSIS
+    >>> manhattan_distances([[2]], [[3]])#doctest:+ELLIPSIS
     array([[ 1.]])
     >>> manhattan_distances([[1, 2], [3, 4]],\
          [[1, 2], [0, 3]])#doctest:+ELLIPSIS
@@ -490,39 +533,21 @@ def manhattan_distances(X, Y=None, sum_over_features=True,
                           X.shape[1], D)
         return D
 
-    temporary_size = X.size * Y.shape[-1]
-    # Convert to bytes
-    temporary_size *= X.itemsize
-    if temporary_size > size_threshold and sum_over_features:
-        # Broadcasting the full thing would be too big: it's on the order
-        # of magnitude of the gigabyte
-        D = np.empty((X.shape[0], Y.shape[0]), dtype=X.dtype)
-        index = 0
-        increment = 1 + int(size_threshold / float(temporary_size) *
-                            X.shape[0])
-        while index < X.shape[0]:
-            this_slice = slice(index, index + increment)
-            tmp = X[this_slice, np.newaxis, :] - Y[np.newaxis, :, :]
-            tmp = np.abs(tmp, tmp)
-            tmp = np.sum(tmp, axis=2)
-            D[this_slice] = tmp
-            index += increment
-    else:
-        D = X[:, np.newaxis, :] - Y[np.newaxis, :, :]
-        D = np.abs(D, D)
-        if sum_over_features:
-            D = np.sum(D, axis=2)
-        else:
-            D = D.reshape((-1, X.shape[1]))
-    return D
+    if sum_over_features:
+        return distance.cdist(X, Y, 'cityblock')
+
+    D = X[:, np.newaxis, :] - Y[np.newaxis, :, :]
+    D = np.abs(D, D)
+    return D.reshape((-1, X.shape[1]))
 
 
 def cosine_distances(X, Y=None):
-    """
-    Compute cosine distance between samples in X and Y.
+    """Compute cosine distance between samples in X and Y.
 
     Cosine distance is defined as 1.0 minus the cosine similarity.
 
+    Read more in the :ref:`User Guide <metrics>`.
+
     Parameters
     ----------
     X : array_like, sparse matrix
@@ -553,6 +578,8 @@ def paired_euclidean_distances(X, Y):
     """
     Computes the paired euclidean distances between X and Y
 
+    Read more in the :ref:`User Guide <metrics>`.
+
     Parameters
     ----------
     X : array-like, shape (n_samples, n_features)
@@ -564,13 +591,14 @@ def paired_euclidean_distances(X, Y):
     distances : ndarray (n_samples, )
     """
     X, Y = check_paired_arrays(X, Y)
-
-    return np.sqrt(((X - Y) ** 2).sum(axis=-1))
+    return row_norms(X - Y)
 
 
 def paired_manhattan_distances(X, Y):
     """Compute the L1 distances between the vectors in X and Y.
 
+    Read more in the :ref:`User Guide <metrics>`.
+
     Parameters
     ----------
     X : array-like, shape (n_samples, n_features)
@@ -582,13 +610,20 @@ def paired_manhattan_distances(X, Y):
     distances : ndarray (n_samples, )
     """
     X, Y = check_paired_arrays(X, Y)
-    return np.abs(X - Y).sum(axis=-1)
+    diff = X - Y
+    if issparse(diff):
+        diff.data = np.abs(diff.data)
+        return np.squeeze(np.array(diff.sum(axis=1)))
+    else:
+        return np.abs(diff).sum(axis=-1)
 
 
 def paired_cosine_distances(X, Y):
     """
     Computes the paired cosine distances between X and Y
 
+    Read more in the :ref:`User Guide <metrics>`.
+
     Parameters
     ----------
     X : array-like, shape (n_samples, n_features)
@@ -605,10 +640,7 @@ def paired_cosine_distances(X, Y):
     euclidean distance if each sample is normalized to unit norm
     """
     X, Y = check_paired_arrays(X, Y)
-
-    X_normalized = normalize(X, copy=True)
-    X_normalized -= normalize(Y, copy=True)
-    return .5 * (X_normalized ** 2).sum(axis=-1)
+    return .5 * row_norms(normalize(X) - normalize(Y), squared=True)
 
 
 PAIRED_DISTANCES = {
@@ -617,8 +649,7 @@ PAIRED_DISTANCES = {
     'l2': paired_euclidean_distances,
     'l1': paired_manhattan_distances,
     'manhattan': paired_manhattan_distances,
-    'cityblock': paired_manhattan_distances,
-    }
+    'cityblock': paired_manhattan_distances}
 
 
 def paired_distances(X, Y, metric="euclidean", **kwds):
@@ -627,9 +658,15 @@ def paired_distances(X, Y, metric="euclidean", **kwds):
 
     Computes the distances between (X[0], Y[0]), (X[1], Y[1]), etc...
 
+    Read more in the :ref:`User Guide <metrics>`.
+
     Parameters
     ----------
-    X, Y : ndarray (n_samples, n_features)
+    X : ndarray (n_samples, n_features)
+        Array 1 for distance computation.
+
+    Y : ndarray (n_samples, n_features)
+        Array 2 for distance computation.
 
     metric : string or callable
         The metric to use when calculating distance between instances in a
@@ -677,6 +714,8 @@ def linear_kernel(X, Y=None):
     """
     Compute the linear kernel between X and Y.
 
+    Read more in the :ref:`User Guide <linear_kernel>`.
+
     Parameters
     ----------
     X : array of shape (n_samples_1, n_features)
@@ -697,13 +736,20 @@ def polynomial_kernel(X, Y=None, degree=3, gamma=None, coef0=1):
 
         K(X, Y) = (gamma <X, Y> + coef0)^degree
 
+    Read more in the :ref:`User Guide <polynomial_kernel>`.
+
     Parameters
     ----------
-    X : array of shape (n_samples_1, n_features)
+    X : ndarray of shape (n_samples_1, n_features)
 
-    Y : array of shape (n_samples_2, n_features)
+    Y : ndarray of shape (n_samples_2, n_features)
 
-    degree : int
+    degree : int, default 3
+
+    gamma : float, default None
+        if None, defaults to 1.0 / n_samples_1
+
+    coef0 : int, default 1
 
     Returns
     -------
@@ -713,7 +759,7 @@ def polynomial_kernel(X, Y=None, degree=3, gamma=None, coef0=1):
     if gamma is None:
         gamma = 1.0 / X.shape[1]
 
-    K = linear_kernel(X, Y)
+    K = safe_sparse_dot(X, Y.T, dense_output=True)
     K *= gamma
     K += coef0
     K **= degree
@@ -726,13 +772,18 @@ def sigmoid_kernel(X, Y=None, gamma=None, coef0=1):
 
         K(X, Y) = tanh(gamma <X, Y> + coef0)
 
+    Read more in the :ref:`User Guide <sigmoid_kernel>`.
+
     Parameters
     ----------
-    X : array of shape (n_samples_1, n_features)
+    X : ndarray of shape (n_samples_1, n_features)
 
-    Y : array of shape (n_samples_2, n_features)
+    Y : ndarray of shape (n_samples_2, n_features)
 
-    degree : int
+    gamma : float, default None
+        If None, defaults to 1.0 / n_samples_1
+
+    coef0 : int, default 1
 
     Returns
     -------
@@ -742,7 +793,7 @@ def sigmoid_kernel(X, Y=None, gamma=None, coef0=1):
     if gamma is None:
         gamma = 1.0 / X.shape[1]
 
-    K = linear_kernel(X, Y)
+    K = safe_sparse_dot(X, Y.T, dense_output=True)
     K *= gamma
     K += coef0
     np.tanh(K, K)   # compute tanh in-place
@@ -757,13 +808,16 @@ def rbf_kernel(X, Y=None, gamma=None):
 
     for each pair of rows x in X and y in Y.
 
+    Read more in the :ref:`User Guide <rbf_kernel>`.
+
     Parameters
     ----------
     X : array of shape (n_samples_X, n_features)
 
     Y : array of shape (n_samples_Y, n_features)
 
-    gamma : float
+    gamma : float, default None
+        If None, defaults to 1.0 / n_samples_X
 
     Returns
     -------
@@ -779,7 +833,41 @@ def rbf_kernel(X, Y=None, gamma=None):
     return K
 
 
-def cosine_similarity(X, Y=None):
+def laplacian_kernel(X, Y=None, gamma=None):
+    """Compute the laplacian kernel between X and Y.
+
+    The laplacian kernel is defined as::
+
+        K(x, y) = exp(-gamma ||x-y||_1)
+
+    for each pair of rows x in X and y in Y.
+    Read more in the :ref:`User Guide <laplacian_kernel>`.
+
+    .. versionadded:: 0.17
+
+    Parameters
+    ----------
+    X : array of shape (n_samples_X, n_features)
+
+    Y : array of shape (n_samples_Y, n_features)
+
+    gamma : float, default None
+        If None, defaults to 1.0 / n_samples_X
+
+    Returns
+    -------
+    kernel_matrix : array of shape (n_samples_X, n_samples_Y)
+    """
+    X, Y = check_pairwise_arrays(X, Y)
+    if gamma is None:
+        gamma = 1.0 / X.shape[1]
+
+    K = -gamma * manhattan_distances(X, Y)
+    np.exp(K, K)    # exponentiate K in-place
+    return K
+
+
+def cosine_similarity(X, Y=None, dense_output=True):
     """Compute cosine similarity between samples in X and Y.
 
     Cosine similarity, or the cosine kernel, computes similarity as the
@@ -789,13 +877,23 @@ def cosine_similarity(X, Y=None):
 
     On L2-normalized data, this function is equivalent to linear_kernel.
 
+    Read more in the :ref:`User Guide <cosine_similarity>`.
+
     Parameters
     ----------
-    X : array_like, sparse matrix
-        with shape (n_samples_X, n_features).
+    X : ndarray or sparse array, shape: (n_samples_X, n_features)
+        Input data.
 
-    Y : array_like, sparse matrix (optional)
-        with shape (n_samples_Y, n_features).
+    Y : ndarray or sparse array, shape: (n_samples_Y, n_features)
+        Input data. If ``None``, the output will be the pairwise
+        similarities between all samples in ``X``.
+
+    dense_output : boolean (optional), default True
+        Whether to return dense output even when the input is sparse. If
+        ``False``, the output is sparse if both input arrays are sparse.
+
+        .. versionadded:: 0.17
+           parameter ``dense_output`` for dense output.
 
     Returns
     -------
@@ -812,7 +910,7 @@ def cosine_similarity(X, Y=None):
     else:
         Y_normalized = normalize(Y, copy=True)
 
-    K = linear_kernel(X_normalized, Y_normalized)
+    K = safe_sparse_dot(X_normalized, Y_normalized.T, dense_output=dense_output)
 
     return K
 
@@ -830,6 +928,8 @@ def additive_chi2_kernel(X, Y=None):
 
     It can be interpreted as a weighted difference per entry.
 
+    Read more in the :ref:`User Guide <chi2_kernel>`.
+
     Notes
     -----
     As the negative of a distance, this kernel is only conditionally positive
@@ -852,7 +952,7 @@ def additive_chi2_kernel(X, Y=None):
       Local features and kernels for classification of texture and object
       categories: A comprehensive study
       International Journal of Computer Vision 2007
-      http://eprints.pascal-network.org/archive/00002309/01/Zhang06-IJCV.pdf
+      http://research.microsoft.com/en-us/um/people/manik/projects/trade-off/papers/ZhangIJCV06.pdf
 
 
     See also
@@ -889,6 +989,8 @@ def chi2_kernel(X, Y=None, gamma=1.):
 
     It can be interpreted as a weighted difference per entry.
 
+    Read more in the :ref:`User Guide <chi2_kernel>`.
+
     Parameters
     ----------
     X : array-like of shape (n_samples_X, n_features)
@@ -908,7 +1010,7 @@ def chi2_kernel(X, Y=None, gamma=1.):
       Local features and kernels for classification of texture and object
       categories: A comprehensive study
       International Journal of Computer Vision 2007
-      http://eprints.pascal-network.org/archive/00002309/01/Zhang06-IJCV.pdf
+      http://research.microsoft.com/en-us/um/people/manik/projects/trade-off/papers/ZhangIJCV06.pdf
 
     See also
     --------
@@ -931,7 +1033,9 @@ PAIRWISE_DISTANCE_FUNCTIONS = {
     'euclidean': euclidean_distances,
     'l2': euclidean_distances,
     'l1': manhattan_distances,
-    'manhattan': manhattan_distances, }
+    'manhattan': manhattan_distances,
+    'precomputed': None,  # HACK: precomputed is always allowed, never called
+}
 
 
 def distance_metrics():
@@ -954,6 +1058,8 @@ def distance_metrics():
     'manhattan'      metrics.pairwise.manhattan_distances
     ============     ====================================
 
+    Read more in the :ref:`User Guide <metrics>`.
+
     """
     return PAIRWISE_DISTANCE_FUNCTIONS
 
@@ -967,13 +1073,51 @@ def _parallel_pairwise(X, Y, func, n_jobs, **kwds):
     if Y is None:
         Y = X
 
+    if n_jobs == 1:
+        # Special case to avoid picklability checks in delayed
+        return func(X, Y, **kwds)
+
+    # TODO: in some cases, backend='threading' may be appropriate
+    fd = delayed(func)
     ret = Parallel(n_jobs=n_jobs, verbose=0)(
-        delayed(func)(X, Y[s], **kwds)
+        fd(X, Y[s], **kwds)
         for s in gen_even_slices(Y.shape[0], n_jobs))
 
     return np.hstack(ret)
 
 
+def _pairwise_callable(X, Y, metric, **kwds):
+    """Handle the callable case for pairwise_{distances,kernels}
+    """
+    X, Y = check_pairwise_arrays(X, Y)
+
+    if X is Y:
+        # Only calculate metric for upper triangle
+        out = np.zeros((X.shape[0], Y.shape[0]), dtype='float')
+        iterator = itertools.combinations(range(X.shape[0]), 2)
+        for i, j in iterator:
+            out[i, j] = metric(X[i], Y[j], **kwds)
+
+        # Make symmetric
+        # NB: out += out.T will produce incorrect results
+        out = out + out.T
+
+        # Calculate diagonal
+        # NB: nonzero diagonals are allowed for both metrics and kernels
+        for i in range(X.shape[0]):
+            x = X[i]
+            out[i, i] = metric(x, x, **kwds)
+
+    else:
+        # Calculate all cells
+        out = np.empty((X.shape[0], Y.shape[0]), dtype='float')
+        iterator = itertools.product(range(X.shape[0]), range(Y.shape[0]))
+        for i, j in iterator:
+            out[i, j] = metric(X[i], Y[j], **kwds)
+
+    return out
+
+
 _VALID_METRICS = ['euclidean', 'l2', 'l1', 'manhattan', 'cityblock',
                   'braycurtis', 'canberra', 'chebyshev', 'correlation',
                   'cosine', 'dice', 'hamming', 'jaccard', 'kulsinski',
@@ -1015,14 +1159,16 @@ def pairwise_distances(X, Y=None, metric="euclidean", n_jobs=1, **kwds):
     scikit-learn, see the __doc__ of the sklearn.pairwise.distance_metrics
     function.
 
+    Read more in the :ref:`User Guide <metrics>`.
+
     Parameters
     ----------
     X : array [n_samples_a, n_samples_a] if metric == "precomputed", or, \
              [n_samples_a, n_features] otherwise
         Array of pairwise distances between samples, or a feature array.
 
-    Y : array [n_samples_b, n_features]
-        A second feature array only if X has shape [n_samples_a, n_features].
+    Y : array [n_samples_b, n_features], optional
+        An optional second feature array. Only allowed if metric != "precomputed".
 
     metric : string, or callable
         The metric to use when calculating distance between instances in a
@@ -1060,50 +1206,47 @@ def pairwise_distances(X, Y=None, metric="euclidean", n_jobs=1, **kwds):
 
     """
     if (metric not in _VALID_METRICS and
-       not callable(metric) and metric != "precomputed"):
+            not callable(metric) and metric != "precomputed"):
         raise ValueError("Unknown metric %s. "
                          "Valid metrics are %s, or 'precomputed', or a "
                          "callable" % (metric, _VALID_METRICS))
 
     if metric == "precomputed":
+        X, _ = check_pairwise_arrays(X, Y, precomputed=True)
         return X
     elif metric in PAIRWISE_DISTANCE_FUNCTIONS:
         func = PAIRWISE_DISTANCE_FUNCTIONS[metric]
-        if n_jobs == 1:
-            return func(X, Y, **kwds)
-        else:
-            return _parallel_pairwise(X, Y, func, n_jobs, **kwds)
     elif callable(metric):
-        # Check matrices first (this is usually done by the metric).
-        X, Y = check_pairwise_arrays(X, Y)
-        n_x, n_y = X.shape[0], Y.shape[0]
-        # Calculate distance for each element in X and Y.
-        # FIXME: can use n_jobs here too
-        # FIXME: np.zeros can be replaced by np.empty
-        D = np.zeros((n_x, n_y), dtype='float')
-        for i in range(n_x):
-            start = 0
-            if X is Y:
-                start = i
-            for j in range(start, n_y):
-                # distance assumed to be symmetric.
-                D[i][j] = metric(X[i], Y[j], **kwds)
-                if X is Y:
-                    D[j][i] = D[i][j]
-        return D
+        func = partial(_pairwise_callable, metric=metric, **kwds)
     else:
-        # Note: the distance module doesn't support sparse matrices!
-        if type(X) is csr_matrix:
+        if issparse(X) or issparse(Y):
             raise TypeError("scipy distance metrics do not"
                             " support sparse matrices.")
-        if Y is None:
+
+        dtype = bool if metric in PAIRWISE_BOOLEAN_FUNCTIONS else None
+
+        X, Y = check_pairwise_arrays(X, Y, dtype=dtype)
+
+        if n_jobs == 1 and X is Y:
             return distance.squareform(distance.pdist(X, metric=metric,
                                                       **kwds))
-        else:
-            if type(Y) is csr_matrix:
-                raise TypeError("scipy distance metrics do not"
-                                " support sparse matrices.")
-            return distance.cdist(X, Y, metric=metric, **kwds)
+        func = partial(distance.cdist, metric=metric, **kwds)
+
+    return _parallel_pairwise(X, Y, func, n_jobs, **kwds)
+
+
+# These distances recquire boolean arrays, when using scipy.spatial.distance
+PAIRWISE_BOOLEAN_FUNCTIONS = [
+    'dice',
+    'jaccard',
+    'kulsinski',
+    'matching',
+    'rogerstanimoto',
+    'russellrao',
+    'sokalmichener',
+    'sokalsneath',
+    'yule',
+]
 
 
 # Helper functions - distance
@@ -1116,6 +1259,7 @@ PAIRWISE_KERNEL_FUNCTIONS = {
     'polynomial': polynomial_kernel,
     'poly': polynomial_kernel,
     'rbf': rbf_kernel,
+    'laplacian': laplacian_kernel,
     'sigmoid': sigmoid_kernel,
     'cosine': cosine_similarity, }
 
@@ -1137,9 +1281,12 @@ def kernel_metrics():
       'poly'            sklearn.pairwise.polynomial_kernel
       'polynomial'      sklearn.pairwise.polynomial_kernel
       'rbf'             sklearn.pairwise.rbf_kernel
+      'laplacian'       sklearn.pairwise.laplacian_kernel
       'sigmoid'         sklearn.pairwise.sigmoid_kernel
       'cosine'          sklearn.pairwise.cosine_similarity
       ===============   ========================================
+
+    Read more in the :ref:`User Guide <metrics>`.
     """
     return PAIRWISE_KERNEL_FUNCTIONS
 
@@ -1153,6 +1300,7 @@ KERNEL_PARAMS = {
     "poly": frozenset(["gamma", "degree", "coef0"]),
     "polynomial": frozenset(["gamma", "degree", "coef0"]),
     "rbf": frozenset(["gamma"]),
+    "laplacian": frozenset(["gamma"]),
     "sigmoid": frozenset(["gamma", "coef0"]),
 }
 
@@ -1175,6 +1323,8 @@ def pairwise_kernels(X, Y=None, metric="linear", filter_params=False,
     Valid values for metric are::
         ['rbf', 'sigmoid', 'polynomial', 'poly', 'linear', 'cosine']
 
+    Read more in the :ref:`User Guide <metrics>`.
+
     Parameters
     ----------
     X : array [n_samples_a, n_samples_a] if metric == "precomputed", or, \
@@ -1223,32 +1373,22 @@ def pairwise_kernels(X, Y=None, metric="linear", filter_params=False,
     If metric is 'precomputed', Y is ignored and X is returned.
 
     """
+    # import GPKernel locally to prevent circular imports
+    from ..gaussian_process.kernels import Kernel as GPKernel
+
     if metric == "precomputed":
+        X, _ = check_pairwise_arrays(X, Y, precomputed=True)
         return X
+    elif isinstance(metric, GPKernel):
+        func = metric.__call__
     elif metric in PAIRWISE_KERNEL_FUNCTIONS:
         if filter_params:
             kwds = dict((k, kwds[k]) for k in kwds
                         if k in KERNEL_PARAMS[metric])
         func = PAIRWISE_KERNEL_FUNCTIONS[metric]
-        if n_jobs == 1:
-            return func(X, Y, **kwds)
-        else:
-            return _parallel_pairwise(X, Y, func, n_jobs, **kwds)
     elif callable(metric):
-        # Check matrices first (this is usually done by the metric).
-        X, Y = check_pairwise_arrays(X, Y)
-        n_x, n_y = X.shape[0], Y.shape[0]
-        # Calculate kernel for each element in X and Y.
-        K = np.zeros((n_x, n_y), dtype='float')
-        for i in range(n_x):
-            start = 0
-            if X is Y:
-                start = i
-            for j in range(start, n_y):
-                # Kernel assumed to be symmetric.
-                K[i][j] = metric(X[i], Y[j], **kwds)
-                if X is Y:
-                    K[j][i] = K[i][j]
-        return K
+        func = partial(_pairwise_callable, metric=metric, **kwds)
     else:
         raise ValueError("Unknown kernel %r" % metric)
+
+    return _parallel_pairwise(X, Y, func, n_jobs, **kwds)
