@@ -19,6 +19,7 @@ from sklearn.utils.testing import assert_equal
 from sklearn.utils.testing import assert_greater
 from sklearn.utils.testing import assert_not_equal
 from sklearn.utils.testing import assert_raises
+from sklearn.utils.testing import assert_raise_message
 from sklearn.utils.testing import assert_true
 from sklearn.utils.testing import ignore_warnings
 
@@ -47,7 +48,7 @@ from sklearn.metrics import recall_score
 from sklearn.metrics import roc_auc_score
 from sklearn.metrics import zero_one_loss
 
-# TODO Curve are currently not coverd by invariance test
+# TODO Curve are currently not covered by invariance test
 # from sklearn.metrics import precision_recall_curve
 # from sklearn.metrics import roc_curve
 
@@ -187,32 +188,54 @@ ALL_METRICS.update(REGRESSION_METRICS)
 # When you add a new metric or functionality, check if a general test
 # is already written.
 
-# Metric undefined with "binary" or "multiclass" input
-METRIC_UNDEFINED_MULTICLASS = [
-    "samples_f0.5_score", "samples_f1_score", "samples_f2_score",
-    "samples_precision_score", "samples_recall_score",
+# Those metrics don't support binary inputs
+METRIC_UNDEFINED_BINARY = [
+    "samples_f0.5_score",
+    "samples_f1_score",
+    "samples_f2_score",
+    "samples_precision_score",
+    "samples_recall_score",
+    "coverage_error",
 
-    # Those metrics don't support multiclass outputs
-    "average_precision_score", "weighted_average_precision_score",
-    "micro_average_precision_score", "macro_average_precision_score",
+    "roc_auc_score",
+    "micro_roc_auc",
+    "weighted_roc_auc",
+    "macro_roc_auc",
+    "samples_roc_auc",
+
+    "average_precision_score",
+    "weighted_average_precision_score",
+    "micro_average_precision_score",
+    "macro_average_precision_score",
     "samples_average_precision_score",
 
-    "label_ranking_average_precision_score",
-
-    "roc_auc_score", "micro_roc_auc", "weighted_roc_auc",
-    "macro_roc_auc",  "samples_roc_auc",
-
-    "coverage_error",
-    "brier_score_loss",
     "label_ranking_loss",
+    "label_ranking_average_precision_score",
 ]
 
+# Those metrics don't support multiclass inputs
+METRIC_UNDEFINED_MULTICLASS = [
+    "brier_score_loss",
+    "matthews_corrcoef_score",
+
+    # with default average='binary', multiclass is prohibited
+    "precision_score",
+    "recall_score",
+    "f1_score",
+    "f2_score",
+    "f0.5_score",
+]
+
+# Metric undefined with "binary" or "multiclass" input
+METRIC_UNDEFINED_BINARY_MULTICLASS = set(METRIC_UNDEFINED_BINARY).union(
+    set(METRIC_UNDEFINED_MULTICLASS))
+
 # Metrics with an "average" argument
 METRICS_WITH_AVERAGING = [
     "precision_score", "recall_score", "f1_score", "f2_score", "f0.5_score"
 ]
 
-# Treshold-based metrics with an "average" argument
+# Threshold-based metrics with an "average" argument
 THRESHOLDED_METRICS_WITH_AVERAGING = [
     "roc_auc_score", "average_precision_score",
 ]
@@ -242,6 +265,8 @@ METRICS_WITH_POS_LABEL = [
 METRICS_WITH_LABELS = [
     "confusion_matrix",
 
+    "hamming_loss",
+
     "precision_score", "recall_score", "f1_score", "f2_score", "f0.5_score",
 
     "weighted_f0.5_score", "weighted_f1_score", "weighted_f2_score",
@@ -285,17 +310,15 @@ MULTILABELS_METRICS = [
     "jaccard_similarity_score", "unnormalized_jaccard_similarity_score",
     "zero_one_loss", "unnormalized_zero_one_loss",
 
-    "precision_score", "recall_score", "f1_score", "f2_score", "f0.5_score",
-
     "weighted_f0.5_score", "weighted_f1_score", "weighted_f2_score",
     "weighted_precision_score", "weighted_recall_score",
 
-    "micro_f0.5_score", "micro_f1_score", "micro_f2_score",
-    "micro_precision_score", "micro_recall_score",
-
     "macro_f0.5_score", "macro_f1_score", "macro_f2_score",
     "macro_precision_score", "macro_recall_score",
 
+    "micro_f0.5_score", "micro_f1_score", "micro_f2_score",
+    "micro_precision_score", "micro_recall_score",
+
     "samples_f0.5_score", "samples_f1_score", "samples_f2_score",
     "samples_precision_score", "samples_recall_score",
 ]
@@ -314,7 +337,11 @@ SYMMETRIC_METRICS = [
     "jaccard_similarity_score", "unnormalized_jaccard_similarity_score",
     "zero_one_loss", "unnormalized_zero_one_loss",
 
-    "f1_score", "weighted_f1_score", "micro_f1_score", "macro_f1_score",
+    "f1_score", "micro_f1_score", "macro_f1_score",
+    "weighted_recall_score",
+    # P = R = F = accuracy in multiclass case
+    "micro_f0.5_score", "micro_f1_score", "micro_f2_score",
+    "micro_precision_score", "micro_recall_score",
 
     "matthews_corrcoef_score", "mean_absolute_error", "mean_squared_error",
     "median_absolute_error",
@@ -331,11 +358,8 @@ NOT_SYMMETRIC_METRICS = [
 
     "precision_score", "recall_score", "f2_score", "f0.5_score",
 
-    "weighted_f0.5_score", "weighted_f2_score", "weighted_precision_score",
-    "weighted_recall_score",
-
-    "micro_f0.5_score", "micro_f2_score", "micro_precision_score",
-    "micro_recall_score",
+    "weighted_f0.5_score", "weighted_f1_score", "weighted_f2_score",
+    "weighted_precision_score",
 
     "macro_f0.5_score", "macro_f2_score", "macro_precision_score",
     "macro_recall_score", "log_loss", "hinge_loss"
@@ -345,9 +369,11 @@ NOT_SYMMETRIC_METRICS = [
 # No Sample weight support
 METRICS_WITHOUT_SAMPLE_WEIGHT = [
     "cohen_kappa_score",
-    "confusion_matrix",
-    "hamming_loss",
-    "matthews_corrcoef_score",
+    "confusion_matrix", # Left this one here because the tests in this file do
+                        # not work for confusion_matrix, as its output is a
+                        # matrix instead of a number. Testing of
+                        # confusion_matrix with sample_weight is in
+                        # test_classification.py
     "median_absolute_error",
 ]
 
@@ -360,10 +386,10 @@ def test_symmetry():
     y_pred = random_state.randint(0, 2, size=(20, ))
 
     # We shouldn't forget any metrics
-    assert_equal(set(SYMMETRIC_METRICS).union(NOT_SYMMETRIC_METRICS,
-                                              THRESHOLDED_METRICS,
-                                              METRIC_UNDEFINED_MULTICLASS),
-                 set(ALL_METRICS))
+    assert_equal(set(SYMMETRIC_METRICS).union(
+        NOT_SYMMETRIC_METRICS, THRESHOLDED_METRICS,
+        METRIC_UNDEFINED_BINARY_MULTICLASS),
+        set(ALL_METRICS))
 
     assert_equal(
         set(SYMMETRIC_METRICS).intersection(set(NOT_SYMMETRIC_METRICS)),
@@ -391,7 +417,7 @@ def test_sample_order_invariance():
     y_true_shuffle, y_pred_shuffle = shuffle(y_true, y_pred, random_state=0)
 
     for name, metric in ALL_METRICS.items():
-        if name in METRIC_UNDEFINED_MULTICLASS:
+        if name in METRIC_UNDEFINED_BINARY_MULTICLASS:
             continue
 
         assert_almost_equal(metric(y_true, y_pred),
@@ -458,7 +484,7 @@ def test_format_invariance_with_1d_vectors():
     y2_row = np.reshape(y2_1d, (1, -1))
 
     for name, metric in ALL_METRICS.items():
-        if name in METRIC_UNDEFINED_MULTICLASS:
+        if name in METRIC_UNDEFINED_BINARY_MULTICLASS:
             continue
 
         measure = metric(y1, y2)
@@ -533,7 +559,7 @@ def test_invariance_string_vs_numbers_labels():
     labels_str = ["eggs", "spam"]
 
     for name, metric in CLASSIFICATION_METRICS.items():
-        if name in METRIC_UNDEFINED_MULTICLASS:
+        if name in METRIC_UNDEFINED_BINARY_MULTICLASS:
             continue
 
         measure_with_number = metric(y1, y2)
@@ -592,6 +618,29 @@ def test_invariance_string_vs_numbers_labels():
             assert_raises(ValueError, metric, y1_str.astype('O'), y2)
 
 
+def test_inf_nan_input():
+    invalids =[([0, 1], [np.inf, np.inf]),
+               ([0, 1], [np.nan, np.nan]),
+               ([0, 1], [np.nan, np.inf])]
+
+    METRICS = dict()
+    METRICS.update(THRESHOLDED_METRICS)
+    METRICS.update(REGRESSION_METRICS)
+
+    for metric in METRICS.values():
+        for y_true, y_score in invalids:
+            assert_raise_message(ValueError,
+                                 "contains NaN, infinity",
+                                 metric, y_true, y_score)
+
+    # Classification metrics all raise a mixed input exception
+    for metric in CLASSIFICATION_METRICS.values():
+        for y_true, y_score in invalids:
+            assert_raise_message(ValueError,
+                                 "Can't handle mix of binary and continuous",
+                                 metric, y_true, y_score)
+
+
 @ignore_warnings
 def check_single_sample(name):
     # Non-regression test: scores should work with a single sample.
@@ -614,7 +663,8 @@ def check_single_sample_multioutput(name):
 
 def test_single_sample():
     for name in ALL_METRICS:
-        if name in METRIC_UNDEFINED_MULTICLASS or name in THRESHOLDED_METRICS:
+        if (name in METRIC_UNDEFINED_BINARY_MULTICLASS or
+                name in THRESHOLDED_METRICS):
             # Those metrics are not always defined with one sample
             # or in multiclass classification
             continue
@@ -916,9 +966,9 @@ def check_sample_weight_invariance(name, metric, y1, y2):
                                  sample_weight=sample_weight.tolist())
     assert_almost_equal(
         weighted_score, weighted_score_list,
-        err_msg="Weighted scores for array and list sample_weight input are "
-                "not equal (%f != %f) for %s" % (
-                    weighted_score, weighted_score_list, name))
+        err_msg=("Weighted scores for array and list "
+                 "sample_weight input are not equal (%f != %f) for %s") % (
+                     weighted_score, weighted_score_list, name))
 
     # check that integer weights is the same as repeated samples
     repeat_weighted_score = metric(
@@ -964,14 +1014,14 @@ def check_sample_weight_invariance(name, metric, y1, y2):
 def test_sample_weight_invariance(n_samples=50):
     random_state = check_random_state(0)
 
-    # binary output
+    # binary
     random_state = check_random_state(0)
     y_true = random_state.randint(0, 2, size=(n_samples, ))
     y_pred = random_state.randint(0, 2, size=(n_samples, ))
     y_score = random_state.random_sample(size=(n_samples,))
     for name in ALL_METRICS:
         if (name in METRICS_WITHOUT_SAMPLE_WEIGHT or
-                name in METRIC_UNDEFINED_MULTICLASS):
+                name in METRIC_UNDEFINED_BINARY):
             continue
         metric = ALL_METRICS[name]
         if name in THRESHOLDED_METRICS:
@@ -986,7 +1036,7 @@ def test_sample_weight_invariance(n_samples=50):
     y_score = random_state.random_sample(size=(n_samples, 5))
     for name in ALL_METRICS:
         if (name in METRICS_WITHOUT_SAMPLE_WEIGHT or
-                name in METRIC_UNDEFINED_MULTICLASS):
+                name in METRIC_UNDEFINED_BINARY_MULTICLASS):
             continue
         metric = ALL_METRICS[name]
         if name in THRESHOLDED_METRICS:
@@ -1019,6 +1069,7 @@ def test_sample_weight_invariance(n_samples=50):
                    y_pred)
 
 
+@ignore_warnings
 def test_no_averaging_labels():
     # test labels argument when not using averaging
     # in multi-class and multi-label cases
@@ -1032,7 +1083,7 @@ def test_no_averaging_labels():
     for name in METRICS_WITH_AVERAGING:
         for y_true, y_pred in [[y_true_multiclass, y_pred_multiclass],
                                [y_true_multilabel, y_pred_multilabel]]:
-            if name not in MULTILABELS_METRICS and y_pred.shape[1] > 0:
+            if name not in MULTILABELS_METRICS and y_pred.ndim > 1:
                 continue
 
             metric = ALL_METRICS[name]
