@@ -29,19 +29,38 @@ case.
 """
 
 # Author: Mathieu Blondel <mathieu@mblondel.org>
+# Author: Hamzeh Alsalhi <93hamsal@gmail.com>
 #
 # License: BSD 3 clause
 
+import array
 import numpy as np
 import warnings
+import scipy.sparse as sp
+import itertools
 
 from .base import BaseEstimator, ClassifierMixin, clone, is_classifier
-from .base import MetaEstimatorMixin
+from .base import MetaEstimatorMixin, is_regressor
 from .preprocessing import LabelBinarizer
 from .metrics.pairwise import euclidean_distances
 from .utils import check_random_state
+from .utils.validation import _num_samples
+from .utils.validation import check_is_fitted
+from .utils.validation import check_X_y
+from .utils.multiclass import (_check_partial_fit_first_call,
+                               check_classification_targets,
+                               _ovr_decision_function)
+from .utils.metaestimators import _safe_split
+
 from .externals.joblib import Parallel
 from .externals.joblib import delayed
+from .externals.six.moves import zip as izip
+
+__all__ = [
+    "OneVsRestClassifier",
+    "OneVsOneClassifier",
+    "OutputCodeClassifier",
+]
 
 
 def _fit_binary(estimator, X, y, classes=None):
@@ -62,8 +81,16 @@ def _fit_binary(estimator, X, y, classes=None):
     return estimator
 
 
+def _partial_fit_binary(estimator, X, y):
+    """Partially fit a single binary estimator."""
+    estimator.partial_fit(X, y, np.array((0, 1)))
+    return estimator
+
+
 def _predict_binary(estimator, X):
     """Make predictions using a single binary estimator."""
+    if is_regressor(estimator):
+        return estimator.predict(X)
     try:
         score = np.ravel(estimator.decision_function(X))
     except (AttributeError, NotImplementedError):
@@ -80,55 +107,25 @@ def _check_estimator(estimator):
                          "decision_function or predict_proba!")
 
 
-def fit_ovr(estimator, X, y, n_jobs=1):
-    """Fit a one-vs-the-rest strategy."""
-    _check_estimator(estimator)
-
-    lb = LabelBinarizer()
-    Y = lb.fit_transform(y)
-
-    estimators = Parallel(n_jobs=n_jobs)(
-        delayed(_fit_binary)(estimator, X, Y[:, i], classes=["not %s" % i, i])
-        for i in range(Y.shape[1]))
-    return estimators, lb
-
-
-def predict_ovr(estimators, label_binarizer, X):
-    """Make predictions using the one-vs-the-rest strategy."""
-    Y = np.array([_predict_binary(e, X) for e in estimators])
-    e = estimators[0]
-    thresh = 0 if hasattr(e, "decision_function") and is_classifier(e) else .5
-    return label_binarizer.inverse_transform(Y.T, threshold=thresh)
-
-
-def predict_proba_ovr(estimators, X, is_multilabel):
-    """Estimate probabilities using the one-vs-the-rest strategy.
-
-    If multilabel is true, returned matrix will not sum to one.  Estimators
-    must have a predict_proba method."""
-
-    # Y[i,j] gives the probability that sample i has the label j.
-    # In the multi-label case, these are not disjoint.
-    Y = np.array([est.predict_proba(X)[:, 1] for est in estimators]).T
-
-    if not is_multilabel:
-        # Then, probabilities should be normalized to 1.
-        Y /= np.sum(Y, axis=1)[:, np.newaxis]
-    return Y
-
-
 class _ConstantPredictor(BaseEstimator):
+
     def fit(self, X, y):
         self.y_ = y
         return self
 
     def predict(self, X):
+        check_is_fitted(self, 'y_')
+
         return np.repeat(self.y_, X.shape[0])
 
     def decision_function(self, X):
+        check_is_fitted(self, 'y_')
+
         return np.repeat(self.y_, X.shape[0])
 
     def predict_proba(self, X):
+        check_is_fitted(self, 'y_')
+
         return np.repeat([np.hstack([1 - self.y_, self.y_])],
                          X.shape[0], axis=0)
 
@@ -152,6 +149,8 @@ class OneVsRestClassifier(BaseEstimator, ClassifierMixin, MetaEstimatorMixin):
     In the multilabel learning literature, OvR is also known as the binary
     relevance method.
 
+    Read more in the :ref:`User Guide <ovr_classification>`.
+
     Parameters
     ----------
     estimator : estimator object
@@ -166,15 +165,15 @@ class OneVsRestClassifier(BaseEstimator, ClassifierMixin, MetaEstimatorMixin):
 
     Attributes
     ----------
-    `estimators_` : list of `n_classes` estimators
+    estimators_ : list of `n_classes` estimators
         Estimators used for predictions.
 
-    `classes_` : array, shape = [`n_classes`]
+    classes_ : array, shape = [`n_classes`]
         Class labels.
-    `label_binarizer_` : LabelBinarizer object
+    label_binarizer_ : LabelBinarizer object
         Object used to transform multiclass labels to binary labels and
         vice-versa.
-    `multilabel_` : boolean
+    multilabel_ : boolean
         Whether a OneVsRestClassifier is a multilabel classifier.
     """
 
@@ -187,10 +186,10 @@ class OneVsRestClassifier(BaseEstimator, ClassifierMixin, MetaEstimatorMixin):
 
         Parameters
         ----------
-        X : {array-like, sparse matrix}, shape = [n_samples, n_features]
+        X : (sparse) array-like, shape = [n_samples, n_features]
             Data.
 
-        y : array-like, shape = [n_samples] or [n_samples, n_classes]
+        y : (sparse) array-like, shape = [n_samples, ], [n_samples, n_classes]
             Multi-class targets. An indicator matrix turns on multilabel
             classification.
 
@@ -198,30 +197,117 @@ class OneVsRestClassifier(BaseEstimator, ClassifierMixin, MetaEstimatorMixin):
         -------
         self
         """
-        self.estimators_, self.label_binarizer_ = fit_ovr(self.estimator, X, y,
-                                                          n_jobs=self.n_jobs)
+        # A sparse LabelBinarizer, with sparse_output=True, has been shown to
+        # outpreform or match a dense label binarizer in all cases and has also
+        # resulted in less or equal memory consumption in the fit_ovr function
+        # overall.
+        self.label_binarizer_ = LabelBinarizer(sparse_output=True)
+        Y = self.label_binarizer_.fit_transform(y)
+        Y = Y.tocsc()
+        self.classes_ = self.label_binarizer_.classes_
+        columns = (col.toarray().ravel() for col in Y.T)
+        # In cases where individual estimators are very fast to train setting
+        # n_jobs > 1 in can results in slower performance due to the overhead
+        # of spawning threads.  See joblib issue #112.
+        self.estimators_ = Parallel(n_jobs=self.n_jobs)(delayed(_fit_binary)(
+            self.estimator, X, column, classes=[
+                "not %s" % self.label_binarizer_.classes_[i],
+                self.label_binarizer_.classes_[i]])
+            for i, column in enumerate(columns))
+
         return self
 
-    def _check_is_fitted(self):
-        if not hasattr(self, "estimators_"):
-            raise ValueError("The object hasn't been fitted yet!")
+    def partial_fit(self, X, y, classes=None):
+        """Partially fit underlying estimators
+
+        Should be used when memory is inefficient to train all data.
+        Chunks of data can be passed in several iteration.
+
+        Parameters
+        ----------
+        X : (sparse) array-like, shape = [n_samples, n_features]
+            Data.
+
+        y : (sparse) array-like, shape = [n_samples, ], [n_samples, n_classes]
+            Multi-class targets. An indicator matrix turns on multilabel
+            classification.
+
+        classes : array, shape (n_classes, )
+            Classes across all calls to partial_fit.
+            Can be obtained via `np.unique(y_all)`, where y_all is the
+            target vector of the entire dataset.
+            This argument is only required in the first call of partial_fit
+            and can be omitted in the subsequent calls.
+
+        Returns
+        -------
+        self
+        """
+        if _check_partial_fit_first_call(self, classes):
+            if (not hasattr(self.estimator, "partial_fit")):
+                raise ValueError("Base estimator {0}, doesn't have partial_fit"
+                                 "method".format(self.estimator))
+            self.estimators_ = [clone(self.estimator) for _ in range
+                                (self.n_classes_)]
+
+        # A sparse LabelBinarizer, with sparse_output=True, has been shown to
+        # outperform or match a dense label binarizer in all cases and has also
+        # resulted in less or equal memory consumption in the fit_ovr function
+        # overall.
+        self.label_binarizer_ = LabelBinarizer(sparse_output=True)
+        Y = self.label_binarizer_.fit_transform(y)
+        Y = Y.tocsc()
+        columns = (col.toarray().ravel() for col in Y.T)
+
+        self.estimators_ = Parallel(n_jobs=self.n_jobs)(delayed(
+            _partial_fit_binary)(self.estimators_[i],
+                                 X, next(columns) if self.classes_[i] in
+                                 self.label_binarizer_.classes_ else
+                                 np.zeros((1, len(y))))
+            for i in range(self.n_classes_))
+
+        return self
 
     def predict(self, X):
         """Predict multi-class targets using underlying estimators.
 
         Parameters
         ----------
-        X : {array-like, sparse matrix}, shape = [n_samples, n_features]
+        X : (sparse) array-like, shape = [n_samples, n_features]
             Data.
 
         Returns
         -------
-        y : array-like, shape = [n_samples]
+        y : (sparse) array-like, shape = [n_samples, ], [n_samples, n_classes].
             Predicted multi-class targets.
         """
-        self._check_is_fitted()
+        check_is_fitted(self, 'estimators_')
+        if (hasattr(self.estimators_[0], "decision_function") and
+                is_classifier(self.estimators_[0])):
+            thresh = 0
+        else:
+            thresh = .5
 
-        return predict_ovr(self.estimators_, self.label_binarizer_, X)
+        n_samples = _num_samples(X)
+        if self.label_binarizer_.y_type_ == "multiclass":
+            maxima = np.empty(n_samples, dtype=float)
+            maxima.fill(-np.inf)
+            argmaxima = np.zeros(n_samples, dtype=int)
+            for i, e in enumerate(self.estimators_):
+                pred = _predict_binary(e, X)
+                np.maximum(maxima, pred, out=maxima)
+                argmaxima[maxima == pred] = i
+            return self.classes_[np.array(argmaxima.T)]
+        else:
+            indices = array.array('i')
+            indptr = array.array('i', [0])
+            for e in self.estimators_:
+                indices.extend(np.where(_predict_binary(e, X) > thresh)[0])
+                indptr.append(len(indices))
+            data = np.ones(len(indices), dtype=int)
+            indicator = sp.csc_matrix((data, indices, indptr),
+                                      shape=(n_samples, len(self.estimators_)))
+            return self.label_binarizer_.inverse_transform(indicator)
 
     def predict_proba(self, X):
         """Probability estimates.
@@ -242,12 +328,24 @@ class OneVsRestClassifier(BaseEstimator, ClassifierMixin, MetaEstimatorMixin):
 
         Returns
         -------
-        T : array-like, shape = [n_samples, n_classes]
+        T : (sparse) array-like, shape = [n_samples, n_classes]
             Returns the probability of the sample for each class in the model,
             where classes are ordered as they are in `self.classes_`.
         """
-        return predict_proba_ovr(self.estimators_, X,
-                                 is_multilabel=self.multilabel_)
+        check_is_fitted(self, 'estimators_')
+        # Y[i, j] gives the probability that sample i has the label j.
+        # In the multi-label case, these are not disjoint.
+        Y = np.array([e.predict_proba(X)[:, 1] for e in self.estimators_]).T
+
+        if len(self.estimators_) == 1:
+            # Only one estimator, but we still want to return probabilities
+            # for two classes.
+            Y = np.concatenate(((1 - Y), Y), axis=1)
+
+        if not self.multilabel_:
+            # Then, probabilities should be normalized to 1.
+            Y /= np.sum(Y, axis=1)[:, np.newaxis]
+        return Y
 
     def decision_function(self, X):
         """Returns the distance of each sample from the decision boundary for
@@ -262,6 +360,7 @@ class OneVsRestClassifier(BaseEstimator, ClassifierMixin, MetaEstimatorMixin):
         -------
         T : array-like, shape = [n_samples, n_classes]
         """
+        check_is_fitted(self, 'estimators_')
         if not hasattr(self.estimators_[0], "decision_function"):
             raise AttributeError(
                 "Base estimator doesn't have a decision_function attribute.")
@@ -271,35 +370,36 @@ class OneVsRestClassifier(BaseEstimator, ClassifierMixin, MetaEstimatorMixin):
     @property
     def multilabel_(self):
         """Whether this is a multilabel classifier"""
-        return self.label_binarizer_.multilabel_
-
-    def score(self, X, y):
-        if self.multilabel_:
-            raise NotImplementedError(
-                "score is not supported for multilabel classifiers")
-        else:
-            return super(OneVsRestClassifier, self).score(X, y)
+        return self.label_binarizer_.y_type_.startswith('multilabel')
 
     @property
-    def classes_(self):
-        return self.label_binarizer_.classes_
+    def n_classes_(self):
+        return len(self.classes_)
 
     @property
     def coef_(self):
-        self._check_is_fitted()
+        check_is_fitted(self, 'estimators_')
         if not hasattr(self.estimators_[0], "coef_"):
             raise AttributeError(
                 "Base estimator doesn't have a coef_ attribute.")
-        return np.array([e.coef_.ravel() for e in self.estimators_])
+        coefs = [e.coef_ for e in self.estimators_]
+        if sp.issparse(coefs[0]):
+            return sp.vstack(coefs)
+        return np.vstack(coefs)
 
     @property
     def intercept_(self):
-        self._check_is_fitted()
+        check_is_fitted(self, 'estimators_')
         if not hasattr(self.estimators_[0], "intercept_"):
             raise AttributeError(
                 "Base estimator doesn't have an intercept_ attribute.")
         return np.array([e.intercept_.ravel() for e in self.estimators_])
 
+    @property
+    def _pairwise(self):
+        """Indicate if wrapped estimator is using a precomputed Gram matrix"""
+        return getattr(self.estimator, "_pairwise", False)
+
 
 def _fit_ovo_binary(estimator, X, y, i, j):
     """Fit a single binary estimator (one-vs-one)."""
@@ -308,50 +408,20 @@ def _fit_ovo_binary(estimator, X, y, i, j):
     y_binary = np.empty(y.shape, np.int)
     y_binary[y == i] = 0
     y_binary[y == j] = 1
-    ind = np.arange(X.shape[0])
-    return _fit_binary(estimator, X[ind[cond]], y_binary, classes=[i, j])
+    indcond = np.arange(X.shape[0])[cond]
+    return _fit_binary(estimator,
+                       _safe_split(estimator, X, None, indices=indcond)[0],
+                       y_binary, classes=[i, j]), indcond
 
 
-def fit_ovo(estimator, X, y, n_jobs=1):
-    """Fit a one-vs-one strategy."""
-    classes = np.unique(y)
-    n_classes = classes.shape[0]
-    estimators = Parallel(n_jobs=n_jobs)(
-        delayed(_fit_ovo_binary)(
-            estimator, X, y, classes[i], classes[j])
-        for i in range(n_classes) for j in range(i + 1, n_classes))
+def _partial_fit_ovo_binary(estimator, X, y, i, j):
+    """Partially fit a single binary estimator(one-vs-one)."""
 
-    return estimators, classes
-
-
-def predict_ovo(estimators, classes, X):
-    """Make predictions using the one-vs-one strategy."""
-    n_samples = X.shape[0]
-    n_classes = classes.shape[0]
-    votes = np.zeros((n_samples, n_classes))
-    scores = np.zeros((n_samples, n_classes))
-
-    k = 0
-    for i in range(n_classes):
-        for j in range(i + 1, n_classes):
-            pred = estimators[k].predict(X)
-            score = _predict_binary(estimators[k], X)
-            scores[:, i] += score
-            scores[:, j] -= score
-            votes[pred == 0, i] += 1
-            votes[pred == 1, j] += 1
-            k += 1
-
-    # find all places with maximum votes per sample
-    maxima = votes == np.max(votes, axis=1)[:, np.newaxis]
-
-    # if there are ties, use scores to break them
-    if np.any(maxima.sum(axis=1) > 1):
-        scores[~maxima] = -np.inf
-        prediction = scores.argmax(axis=1)
-    else:
-        prediction = votes.argmax(axis=1)
-    return classes[prediction]
+    cond = np.logical_or(y == i, y == j)
+    y = y[cond]
+    y_binary = np.zeros_like(y)
+    y_binary[y == j] = 1
+    return _partial_fit_binary(estimator, X[cond], y_binary)
 
 
 class OneVsOneClassifier(BaseEstimator, ClassifierMixin, MetaEstimatorMixin):
@@ -367,6 +437,8 @@ class OneVsOneClassifier(BaseEstimator, ClassifierMixin, MetaEstimatorMixin):
     a small subset of the data whereas, with one-vs-the-rest, the complete
     dataset is used `n_classes` times.
 
+    Read more in the :ref:`User Guide <ovo_classification>`.
+
     Parameters
     ----------
     estimator : estimator object
@@ -381,10 +453,10 @@ class OneVsOneClassifier(BaseEstimator, ClassifierMixin, MetaEstimatorMixin):
 
     Attributes
     ----------
-    `estimators_` : list of `n_classes * (n_classes - 1) / 2` estimators
+    estimators_ : list of `n_classes * (n_classes - 1) / 2` estimators
         Estimators used for predictions.
 
-    `classes_` : numpy array of shape [n_classes]
+    classes_ : numpy array of shape [n_classes]
         Array containing labels.
     """
 
@@ -397,26 +469,90 @@ class OneVsOneClassifier(BaseEstimator, ClassifierMixin, MetaEstimatorMixin):
 
         Parameters
         ----------
-        X : {array-like, sparse matrix}, shape = [n_samples, n_features]
+        X : (sparse) array-like, shape = [n_samples, n_features]
             Data.
 
-        y : numpy array of shape [n_samples]
+        y : array-like, shape = [n_samples]
             Multi-class targets.
 
         Returns
         -------
         self
         """
-        self.estimators_, self.classes_ = fit_ovo(self.estimator, X, y,
-                                                  self.n_jobs)
+        X, y = check_X_y(X, y, accept_sparse=['csr', 'csc'])
+
+        self.classes_ = np.unique(y)
+        n_classes = self.classes_.shape[0]
+        estimators_indices = list(zip(*(Parallel(n_jobs=self.n_jobs)(
+            delayed(_fit_ovo_binary)
+            (self.estimator, X, y, self.classes_[i], self.classes_[j])
+            for i in range(n_classes) for j in range(i + 1, n_classes)))))
+
+        self.estimators_ = estimators_indices[0]
+        try:
+            self.pairwise_indices_ = estimators_indices[1] \
+                                     if self._pairwise else None
+        except AttributeError:
+            self.pairwise_indices_ = None
+
         return self
 
-    def predict(self, X):
-        """Predict multi-class targets using underlying estimators.
+    def partial_fit(self, X, y, classes=None):
+        """Partially fit underlying estimators
+
+        Should be used when memory is inefficient to train all data. Chunks
+        of data can be passed in several iteration, where the first call
+        should have an array of all target variables.
+
 
         Parameters
         ----------
-        X : {array-like, sparse matrix}, shape = [n_samples, n_features]
+        X : (sparse) array-like, shape = [n_samples, n_features]
+            Data.
+
+        y : array-like, shape = [n_samples]
+            Multi-class targets.
+
+        classes : array, shape (n_classes, )
+            Classes across all calls to partial_fit.
+            Can be obtained via `np.unique(y_all)`, where y_all is the
+            target vector of the entire dataset.
+            This argument is only required in the first call of partial_fit
+            and can be omitted in the subsequent calls.
+
+        Returns
+        -------
+        self
+        """
+        if _check_partial_fit_first_call(self, classes):
+            self.estimators_ = [clone(self.estimator) for i in
+                                range(self.n_classes_ *
+                                      (self.n_classes_ - 1) // 2)]
+
+        X, y = check_X_y(X, y, accept_sparse=['csr', 'csc'])
+        check_classification_targets(y)
+        combinations = itertools.combinations(range(self.n_classes_), 2)
+        self.estimators_ = Parallel(
+            n_jobs=self.n_jobs)(
+                delayed(_partial_fit_ovo_binary)(
+                    estimator, X, y, self.classes_[i], self.classes_[j])
+                for estimator, (i, j) in izip(
+                        self.estimators_, (combinations)))
+
+        self.pairwise_indices_ = None
+
+        return self
+
+    def predict(self, X):
+        """Estimate the best class label for each sample in X.
+
+        This is implemented as ``argmax(decision_function(X), axis=1)`` which
+        will return the label of the class with most votes by estimators
+        predicting the outcome of a decision for each possible class pair.
+
+        Parameters
+        ----------
+        X : (sparse) array-like, shape = [n_samples, n_features]
             Data.
 
         Returns
@@ -424,75 +560,50 @@ class OneVsOneClassifier(BaseEstimator, ClassifierMixin, MetaEstimatorMixin):
         y : numpy array of shape [n_samples]
             Predicted multi-class targets.
         """
-        if not hasattr(self, "estimators_"):
-            raise ValueError("The object hasn't been fitted yet!")
+        Y = self.decision_function(X)
+        return self.classes_[Y.argmax(axis=1)]
 
-        return predict_ovo(self.estimators_, self.classes_, X)
+    def decision_function(self, X):
+        """Decision function for the OneVsOneClassifier.
 
+        The decision values for the samples are computed by adding the
+        normalized sum of pair-wise classification confidence levels to the
+        votes in order to disambiguate between the decision values when the
+        votes for all the classes are equal leading to a tie.
 
-def fit_ecoc(estimator, X, y, code_size=1.5, random_state=None, n_jobs=1):
-    """
-    Fit an error-correcting output-code strategy.
+        Parameters
+        ----------
+        X : array-like, shape = [n_samples, n_features]
 
-    Parameters
-    ----------
-    estimator : estimator object
-        An estimator object implementing `fit` and one of `decision_function`
-        or `predict_proba`.
+        Returns
+        -------
+        Y : array-like, shape = [n_samples, n_classes]
+        """
+        check_is_fitted(self, 'estimators_')
 
-    code_size : float, optional
-        Percentage of the number of classes to be used to create the code book.
+        indices = self.pairwise_indices_
+        if indices is None:
+            Xs = [X] * len(self.estimators_)
+        else:
+            Xs = [X[:, idx] for idx in indices]
 
-    random_state : numpy.RandomState, optional
-        The generator used to initialize the codebook. Defaults to
-        numpy.random.
+        predictions = np.vstack([est.predict(Xi)
+                                 for est, Xi in zip(self.estimators_, Xs)]).T
+        confidences = np.vstack([_predict_binary(est, Xi)
+                                 for est, Xi in zip(self.estimators_, Xs)]).T
+        Y = _ovr_decision_function(predictions,
+                                   confidences, len(self.classes_))
 
+        return Y
 
-    Returns
-    --------
-    estimators : list of `int(n_classes * code_size)` estimators
-        Estimators used for predictions.
+    @property
+    def n_classes_(self):
+        return len(self.classes_)
 
-    classes : numpy array of shape [n_classes]
-        Array containing labels.
-
-    `code_book_`: numpy array of shape [n_classes, code_size]
-        Binary array containing the code of each class.
-    """
-    _check_estimator(estimator)
-    random_state = check_random_state(random_state)
-
-    classes = np.unique(y)
-    n_classes = classes.shape[0]
-    code_size = int(n_classes * code_size)
-
-    # FIXME: there are more elaborate methods than generating the codebook
-    # randomly.
-    code_book = random_state.random_sample((n_classes, code_size))
-    code_book[code_book > 0.5] = 1
-
-    if hasattr(estimator, "decision_function"):
-        code_book[code_book != 1] = -1
-    else:
-        code_book[code_book != 1] = 0
-
-    cls_idx = dict((c, i) for i, c in enumerate(classes))
-
-    Y = np.array([code_book[cls_idx[y[i]]] for i in range(X.shape[0])],
-                 dtype=np.int)
-
-    estimators = Parallel(n_jobs=n_jobs)(
-        delayed(_fit_binary)(estimator, X, Y[:, i])
-        for i in range(Y.shape[1]))
-
-    return estimators, classes, code_book
-
-
-def predict_ecoc(estimators, classes, code_book, X):
-    """Make predictions using the error-correcting output-code strategy."""
-    Y = np.array([_predict_binary(e, X) for e in estimators]).T
-    pred = euclidean_distances(Y, code_book).argmin(axis=1)
-    return classes[pred]
+    @property
+    def _pairwise(self):
+        """Indicate if wrapped estimator is using a precomputed Gram matrix"""
+        return getattr(self.estimator, "_pairwise", False)
 
 
 class OutputCodeClassifier(BaseEstimator, ClassifierMixin, MetaEstimatorMixin):
@@ -507,6 +618,8 @@ class OutputCodeClassifier(BaseEstimator, ClassifierMixin, MetaEstimatorMixin):
     for compressing the model (0 < code_size < 1) or for making the model more
     robust to errors (code_size > 1). See the documentation for more details.
 
+    Read more in the :ref:`User Guide <ecoc>`.
+
     Parameters
     ----------
     estimator : estimator object
@@ -531,13 +644,13 @@ class OutputCodeClassifier(BaseEstimator, ClassifierMixin, MetaEstimatorMixin):
 
     Attributes
     ----------
-    `estimators_` : list of `int(n_classes * code_size)` estimators
+    estimators_ : list of `int(n_classes * code_size)` estimators
         Estimators used for predictions.
 
-    `classes_` : numpy array of shape [n_classes]
+    classes_ : numpy array of shape [n_classes]
         Array containing labels.
 
-    `code_book_` : numpy array of shape [n_classes, code_size]
+    code_book_ : numpy array of shape [n_classes, code_size]
         Binary array containing the code of each class.
 
     References
@@ -560,9 +673,6 @@ class OutputCodeClassifier(BaseEstimator, ClassifierMixin, MetaEstimatorMixin):
     """
 
     def __init__(self, estimator, code_size=1.5, random_state=None, n_jobs=1):
-        if (code_size <= 0):
-            raise ValueError("code_size should be greater than 0!")
-
         self.estimator = estimator
         self.code_size = code_size
         self.random_state = random_state
@@ -573,7 +683,7 @@ class OutputCodeClassifier(BaseEstimator, ClassifierMixin, MetaEstimatorMixin):
 
         Parameters
         ----------
-        X : {array-like, sparse matrix}, shape = [n_samples, n_features]
+        X : (sparse) array-like, shape = [n_samples, n_features]
             Data.
 
         y : numpy array of shape [n_samples]
@@ -583,9 +693,36 @@ class OutputCodeClassifier(BaseEstimator, ClassifierMixin, MetaEstimatorMixin):
         -------
         self
         """
-        self.estimators_, self.classes_, self.code_book_ = \
-            fit_ecoc(self.estimator, X, y, self.code_size, self.random_state,
-                     self.n_jobs)
+        if self.code_size <= 0:
+            raise ValueError("code_size should be greater than 0, got {1}"
+                             "".format(self.code_size))
+
+        _check_estimator(self.estimator)
+        random_state = check_random_state(self.random_state)
+
+        self.classes_ = np.unique(y)
+        n_classes = self.classes_.shape[0]
+        code_size_ = int(n_classes * self.code_size)
+
+        # FIXME: there are more elaborate methods than generating the codebook
+        # randomly.
+        self.code_book_ = random_state.random_sample((n_classes, code_size_))
+        self.code_book_[self.code_book_ > 0.5] = 1
+
+        if hasattr(self.estimator, "decision_function"):
+            self.code_book_[self.code_book_ != 1] = -1
+        else:
+            self.code_book_[self.code_book_ != 1] = 0
+
+        classes_index = dict((c, i) for i, c in enumerate(self.classes_))
+
+        Y = np.array([self.code_book_[classes_index[y[i]]]
+                      for i in range(X.shape[0])], dtype=np.int)
+
+        self.estimators_ = Parallel(n_jobs=self.n_jobs)(
+            delayed(_fit_binary)(self.estimator, X, Y[:, i])
+            for i in range(Y.shape[1]))
+
         return self
 
     def predict(self, X):
@@ -593,7 +730,7 @@ class OutputCodeClassifier(BaseEstimator, ClassifierMixin, MetaEstimatorMixin):
 
         Parameters
         ----------
-        X : {array-like, sparse matrix}, shape = [n_samples, n_features]
+        X : (sparse) array-like, shape = [n_samples, n_features]
             Data.
 
         Returns
@@ -601,8 +738,7 @@ class OutputCodeClassifier(BaseEstimator, ClassifierMixin, MetaEstimatorMixin):
         y : numpy array of shape [n_samples]
             Predicted multi-class targets.
         """
-        if not hasattr(self, "estimators_"):
-            raise ValueError("The object hasn't been fitted yet!")
-
-        return predict_ecoc(self.estimators_, self.classes_,
-                            self.code_book_, X)
+        check_is_fitted(self, 'estimators_')
+        Y = np.array([_predict_binary(e, X) for e in self.estimators_]).T
+        pred = euclidean_distances(Y, self.code_book_).argmin(axis=1)
+        return self.classes_[pred]
