@@ -14,41 +14,45 @@ def configuration(parent_package='', top_path=None):
 
     config = Configuration('sklearn', parent_package, top_path)
 
+    # submodules with build utilities
     config.add_subpackage('__check_build')
-    config.add_subpackage('svm')
-    config.add_subpackage('datasets')
-    config.add_subpackage('datasets/tests')
-    config.add_subpackage('feature_extraction')
-    config.add_subpackage('feature_extraction/tests')
-    config.add_subpackage('cluster')
-    config.add_subpackage('cluster/tests')
+    config.add_subpackage('_build_utils')
+
+    # submodules which do not have their own setup.py
+    # we must manually add sub-submodules & tests
     config.add_subpackage('covariance')
     config.add_subpackage('covariance/tests')
     config.add_subpackage('cross_decomposition')
-    config.add_subpackage('decomposition')
-    config.add_subpackage('decomposition/tests')
-    config.add_subpackage("ensemble")
-    config.add_subpackage("ensemble/tests")
+    config.add_subpackage('cross_decomposition/tests')
     config.add_subpackage('feature_selection')
     config.add_subpackage('feature_selection/tests')
-    config.add_subpackage('utils')
-    config.add_subpackage('utils/tests')
-    config.add_subpackage('externals')
-    config.add_subpackage('mixture')
-    config.add_subpackage('mixture/tests')
     config.add_subpackage('gaussian_process')
     config.add_subpackage('gaussian_process/tests')
-    config.add_subpackage('neighbors')
+    config.add_subpackage('mixture')
+    config.add_subpackage('mixture/tests')
+    config.add_subpackage('model_selection')
+    config.add_subpackage('model_selection/tests')
     config.add_subpackage('neural_network')
+    config.add_subpackage('neural_network/tests')
     config.add_subpackage('preprocessing')
+    config.add_subpackage('preprocessing/tests')
+    config.add_subpackage('semi_supervised')
+    config.add_subpackage('semi_supervised/tests')
+
+    # submodules which have their own setup.py
+    # leave out "linear_model" and "utils" for now; add them after cblas below
+    config.add_subpackage('cluster')
+    config.add_subpackage('datasets')
+    config.add_subpackage('decomposition')
+    config.add_subpackage('ensemble')
+    config.add_subpackage('externals')
+    config.add_subpackage('feature_extraction')
     config.add_subpackage('manifold')
     config.add_subpackage('metrics')
-    config.add_subpackage('semi_supervised')
-    config.add_subpackage("tree")
-    config.add_subpackage("tree/tests")
-    config.add_subpackage('metrics/tests')
     config.add_subpackage('metrics/cluster')
-    config.add_subpackage('metrics/cluster/tests')
+    config.add_subpackage('neighbors')
+    config.add_subpackage('tree')
+    config.add_subpackage('svm')
 
     # add cython extension module for isotonic regression
     config.add_extension(
