@@ -4,7 +4,7 @@ Testing for the tree module (sklearn.tree).
 import pickle
 from functools import partial
 from itertools import product
-import platform
+import struct
 
 import numpy as np
 from scipy.sparse import csc_matrix
@@ -25,13 +25,15 @@ from sklearn.utils.testing import assert_raises
 from sklearn.utils.testing import assert_greater
 from sklearn.utils.testing import assert_greater_equal
 from sklearn.utils.testing import assert_less
+from sklearn.utils.testing import assert_less_equal
 from sklearn.utils.testing import assert_true
 from sklearn.utils.testing import assert_warns
 from sklearn.utils.testing import raises
+from sklearn.utils.testing import ignore_warnings
 
 from sklearn.utils.validation import check_random_state
-from sklearn.utils.validation import NotFittedError
-from sklearn.utils.testing import ignore_warnings
+
+from sklearn.exceptions import NotFittedError
 
 from sklearn.tree import DecisionTreeClassifier
 from sklearn.tree import DecisionTreeRegressor
@@ -39,15 +41,13 @@ from sklearn.tree import ExtraTreeClassifier
 from sklearn.tree import ExtraTreeRegressor
 
 from sklearn import tree
-from sklearn.tree.tree import SPARSE_SPLITTERS
 from sklearn.tree._tree import TREE_LEAF
 from sklearn import datasets
 
-from sklearn.preprocessing._weights import _balance_weights
-
+from sklearn.utils import compute_sample_weight
 
 CLF_CRITERIONS = ("gini", "entropy")
-REG_CRITERIONS = ("mse", )
+REG_CRITERIONS = ("mse", "mae")
 
 CLF_TREES = {
     "DecisionTreeClassifier": DecisionTreeClassifier,
@@ -507,6 +507,8 @@ def test_error():
     for name, TreeEstimator in ALL_TREES.items():
         # Invalid values for parameters
         assert_raises(ValueError, TreeEstimator(min_samples_leaf=-1).fit, X, y)
+        assert_raises(ValueError, TreeEstimator(min_samples_leaf=.6).fit, X, y)
+        assert_raises(ValueError, TreeEstimator(min_samples_leaf=0.).fit, X, y)
         assert_raises(ValueError,
                       TreeEstimator(min_weight_fraction_leaf=-1).fit,
                       X, y)
@@ -515,8 +517,13 @@ def test_error():
                       X, y)
         assert_raises(ValueError, TreeEstimator(min_samples_split=-1).fit,
                       X, y)
+        assert_raises(ValueError, TreeEstimator(min_samples_split=0.0).fit,
+                      X, y)
+        assert_raises(ValueError, TreeEstimator(min_samples_split=1.1).fit,
+                      X, y)
         assert_raises(ValueError, TreeEstimator(max_depth=-1).fit, X, y)
         assert_raises(ValueError, TreeEstimator(max_features=42).fit, X, y)
+        assert_raises(ValueError, TreeEstimator(min_impurity_split=-1.0).fit, X, y)
 
         # Wrong dimensions
         est = TreeEstimator()
@@ -556,6 +563,40 @@ def test_error():
         assert_raises(NotFittedError, est.apply, T)
 
 
+def test_min_samples_split():
+    """Test min_samples_split parameter"""
+    X = np.asfortranarray(iris.data.astype(tree._tree.DTYPE))
+    y = iris.target
+
+    # test both DepthFirstTreeBuilder and BestFirstTreeBuilder
+    # by setting max_leaf_nodes
+    for max_leaf_nodes, name in product((None, 1000), ALL_TREES.keys()):
+        TreeEstimator = ALL_TREES[name]
+
+        # test for integer parameter
+        est = TreeEstimator(min_samples_split=10,
+                            max_leaf_nodes=max_leaf_nodes,
+                            random_state=0)
+        est.fit(X, y)
+        # count samples on nodes, -1 means it is a leaf
+        node_samples = est.tree_.n_node_samples[est.tree_.children_left != -1]
+
+        assert_greater(np.min(node_samples), 9,
+                       "Failed with {0}".format(name))
+
+        # test for float parameter
+        est = TreeEstimator(min_samples_split=0.2,
+                            max_leaf_nodes=max_leaf_nodes,
+                            random_state=0)
+        est.fit(X, y)
+        # count samples on nodes, -1 means it is a leaf
+        node_samples = est.tree_.n_node_samples[est.tree_.children_left != -1]
+
+        assert_greater(np.min(node_samples), 9,
+                       "Failed with {0}".format(name))
+
+
+
 def test_min_samples_leaf():
     # Test if leaves contain more than leaf_count training examples
     X = np.asfortranarray(iris.data.astype(tree._tree.DTYPE))
@@ -563,18 +604,32 @@ def test_min_samples_leaf():
 
     # test both DepthFirstTreeBuilder and BestFirstTreeBuilder
     # by setting max_leaf_nodes
-    for max_leaf_nodes in (None, 1000):
-        for name, TreeEstimator in ALL_TREES.items():
-            est = TreeEstimator(min_samples_leaf=5,
-                                max_leaf_nodes=max_leaf_nodes,
-                                random_state=0)
-            est.fit(X, y)
-            out = est.tree_.apply(X)
-            node_counts = np.bincount(out)
-            # drop inner nodes
-            leaf_count = node_counts[node_counts != 0]
-            assert_greater(np.min(leaf_count), 4,
-                           "Failed with {0}".format(name))
+    for max_leaf_nodes, name in product((None, 1000), ALL_TREES.keys()):
+        TreeEstimator = ALL_TREES[name]
+
+        # test integer parameter
+        est = TreeEstimator(min_samples_leaf=5,
+                            max_leaf_nodes=max_leaf_nodes,
+                            random_state=0)
+        est.fit(X, y)
+        out = est.tree_.apply(X)
+        node_counts = np.bincount(out)
+        # drop inner nodes
+        leaf_count = node_counts[node_counts != 0]
+        assert_greater(np.min(leaf_count), 4,
+                       "Failed with {0}".format(name))
+
+        # test float parameter
+        est = TreeEstimator(min_samples_leaf=0.1,
+                            max_leaf_nodes=max_leaf_nodes,
+                            random_state=0)
+        est.fit(X, y)
+        out = est.tree_.apply(X)
+        node_counts = np.bincount(out)
+        # drop inner nodes
+        leaf_count = node_counts[node_counts != 0]
+        assert_greater(np.min(leaf_count), 4,
+                       "Failed with {0}".format(name))
 
 
 def check_min_weight_fraction_leaf(name, datasets, sparse=False):
@@ -626,33 +681,85 @@ def test_min_weight_fraction_leaf():
         yield check_min_weight_fraction_leaf, name, "multilabel", True
 
 
+def test_min_impurity_split():
+    # test if min_impurity_split creates leaves with impurity
+    # [0, min_impurity_split) when min_samples_leaf = 1 and
+    # min_samples_split = 2.
+    X = np.asfortranarray(iris.data.astype(tree._tree.DTYPE))
+    y = iris.target
+
+    # test both DepthFirstTreeBuilder and BestFirstTreeBuilder
+    # by setting max_leaf_nodes
+    for max_leaf_nodes, name in product((None, 1000), ALL_TREES.keys()):
+        TreeEstimator = ALL_TREES[name]
+        min_impurity_split = .5
+
+        # verify leaf nodes without min_impurity_split less than
+        # impurity 1e-7
+        est = TreeEstimator(max_leaf_nodes=max_leaf_nodes,
+                            random_state=0)
+        assert_less_equal(est.min_impurity_split, 1e-7,
+                     "Failed, min_impurity_split = {0} > 1e-7".format(
+                         est.min_impurity_split))
+        est.fit(X, y)
+        for node in range(est.tree_.node_count):
+            if (est.tree_.children_left[node] == TREE_LEAF or
+                est.tree_.children_right[node] == TREE_LEAF):
+                assert_equal(est.tree_.impurity[node], 0.,
+                             "Failed with {0} "
+                             "min_impurity_split={1}".format(
+                                 est.tree_.impurity[node],
+                                 est.min_impurity_split))
+
+        # verify leaf nodes have impurity [0,min_impurity_split] when using min_impurity_split
+        est = TreeEstimator(max_leaf_nodes=max_leaf_nodes,
+                            min_impurity_split=min_impurity_split,
+                            random_state=0)
+        est.fit(X, y)
+        for node in range(est.tree_.node_count):
+            if (est.tree_.children_left[node] == TREE_LEAF or
+                est.tree_.children_right[node] == TREE_LEAF):
+                assert_greater_equal(est.tree_.impurity[node], 0,
+                                     "Failed with {0}, "
+                                     "min_impurity_split={1}".format(
+                                         est.tree_.impurity[node],
+                                         est.min_impurity_split))
+                assert_less_equal(est.tree_.impurity[node], min_impurity_split,
+                                  "Failed with {0}, "
+                                  "min_impurity_split={1}".format(
+                                      est.tree_.impurity[node],
+                                      est.min_impurity_split))
+
+
 def test_pickle():
-    # Check that tree estimator are pickable
-    for name, TreeClassifier in CLF_TREES.items():
-        clf = TreeClassifier(random_state=0)
-        clf.fit(iris.data, iris.target)
-        score = clf.score(iris.data, iris.target)
 
-        serialized_object = pickle.dumps(clf)
-        clf2 = pickle.loads(serialized_object)
-        assert_equal(type(clf2), clf.__class__)
-        score2 = clf2.score(iris.data, iris.target)
-        assert_equal(score, score2, "Failed to generate same score "
-                                    "after pickling (classification) "
-                                    "with {0}".format(name))
+    for name, TreeEstimator in ALL_TREES.items():
+        if "Classifier" in name:
+            X, y = iris.data, iris.target
+        else:
+            X, y = boston.data, boston.target
 
-    for name, TreeRegressor in REG_TREES.items():
-        reg = TreeRegressor(random_state=0)
-        reg.fit(boston.data, boston.target)
-        score = reg.score(boston.data, boston.target)
+        est = TreeEstimator(random_state=0)
+        est.fit(X, y)
+        score = est.score(X, y)
+        fitted_attribute = dict()
+        for attribute in ["max_depth", "node_count", "capacity"]:
+            fitted_attribute[attribute] = getattr(est.tree_, attribute)
+
+        serialized_object = pickle.dumps(est)
+        est2 = pickle.loads(serialized_object)
+        assert_equal(type(est2), est.__class__)
+        score2 = est2.score(X, y)
+        assert_equal(score, score2,
+                     "Failed to generate same score  after pickling "
+                     "with {0}".format(name))
+
+        for attribute in fitted_attribute:
+            assert_equal(getattr(est2.tree_, attribute),
+                         fitted_attribute[attribute],
+                         "Failed to generate same attribute {0} after "
+                         "pickling with {1}".format(attribute, name))
 
-        serialized_object = pickle.dumps(reg)
-        reg2 = pickle.loads(serialized_object)
-        assert_equal(type(reg2), reg.__class__)
-        score2 = reg2.score(boston.data, boston.target)
-        assert_equal(score, score2, "Failed to generate same score "
-                                    "after pickling (regression) "
-                                    "with {0}".format(name))
 
 
 def test_multioutput():
@@ -735,7 +842,7 @@ def test_unbalanced_iris():
     # Check class rebalancing.
     unbalanced_X = iris.data[:125]
     unbalanced_y = iris.target[:125]
-    sample_weight = _balance_weights(unbalanced_y)
+    sample_weight = compute_sample_weight("balanced", unbalanced_y)
 
     for name, TreeClassifier in CLF_TREES.items():
         clf = TreeClassifier(random_state=0)
@@ -950,7 +1057,7 @@ def test_max_leaf_nodes():
 
 
 def test_max_leaf_nodes_max_depth():
-    # Test preceedence of max_leaf_nodes over max_depth.
+    # Test precedence of max_leaf_nodes over max_depth.
     X, y = datasets.make_hastie_10_2(n_samples=100, random_state=1)
     k = 4
     for name, TreeEstimator in ALL_TREES.items():
@@ -964,9 +1071,9 @@ def test_arrays_persist():
     # non-regression for #2726
     for attr in ['n_classes', 'value', 'children_left', 'children_right',
                  'threshold', 'impurity', 'feature', 'n_node_samples']:
-        value = getattr(DecisionTreeClassifier().fit([[0]], [0]).tree_, attr)
+        value = getattr(DecisionTreeClassifier().fit([[0], [1]], [0, 1]).tree_, attr)
         # if pointing to freed memory, contents may be arbitrary
-        assert_true(-2 <= value.flat[0] < 2,
+        assert_true(-3 <= value.flat[0] < 3,
                     'Array points to arbitrary memory')
 
 
@@ -1014,7 +1121,7 @@ def test_realloc():
 
 
 def test_huge_allocations():
-    n_bits = int(platform.architecture()[0].rstrip('bit'))
+    n_bits = 8 * struct.calcsize("P")
 
     X = np.random.randn(10, 2)
     y = np.random.randint(0, 2, 10)
@@ -1220,6 +1327,14 @@ def check_explicit_sparse_zeros(tree, max_depth=3,
         assert_array_almost_equal(s.tree_.apply(X1), d.tree_.apply(X2))
         assert_array_almost_equal(s.apply(X1), d.apply(X2))
         assert_array_almost_equal(s.apply(X1), s.tree_.apply(X1))
+
+        assert_array_almost_equal(s.tree_.decision_path(X1).toarray(),
+                                  d.tree_.decision_path(X2).toarray())
+        assert_array_almost_equal(s.decision_path(X1).toarray(),
+                                  d.decision_path(X2).toarray())
+        assert_array_almost_equal(s.decision_path(X1).toarray(),
+                                  s.tree_.decision_path(X1).toarray())
+
         assert_array_almost_equal(s.predict(X1), d.predict(X2))
 
         if tree in CLF_TREES:
@@ -1309,10 +1424,11 @@ def test_public_apply():
 
 
 def check_presort_sparse(est, X, y):
-    assert_raises(ValueError, est.fit, X, y )
+    assert_raises(ValueError, est.fit, X, y)
+
 
 def test_presort_sparse():
-    ests = (DecisionTreeClassifier(presort=True), 
+    ests = (DecisionTreeClassifier(presort=True),
             DecisionTreeRegressor(presort=True))
     sparse_matrices = (csr_matrix, csc_matrix, coo_matrix)
 
@@ -1324,3 +1440,69 @@ def test_presort_sparse():
 
     for est, sparse_matrix in product(ests, sparse_matrices):
         yield check_presort_sparse, est, sparse_matrix(X), y
+
+
+def test_decision_path_hardcoded():
+    X = iris.data
+    y = iris.target
+    est = DecisionTreeClassifier(random_state=0, max_depth=1).fit(X, y)
+    node_indicator = est.decision_path(X[:2]).toarray()
+    assert_array_equal(node_indicator, [[1, 1, 0], [1, 0, 1]])
+
+
+def check_decision_path(name):
+    X = iris.data
+    y = iris.target
+    n_samples = X.shape[0]
+
+    TreeEstimator = ALL_TREES[name]
+    est = TreeEstimator(random_state=0, max_depth=2)
+    est.fit(X, y)
+
+    node_indicator_csr = est.decision_path(X)
+    node_indicator = node_indicator_csr.toarray()
+    assert_equal(node_indicator.shape, (n_samples, est.tree_.node_count))
+
+    # Assert that leaves index are correct
+    leaves = est.apply(X)
+    leave_indicator = [node_indicator[i, j] for i, j in enumerate(leaves)]
+    assert_array_almost_equal(leave_indicator, np.ones(shape=n_samples))
+
+    # Ensure only one leave node per sample
+    all_leaves = est.tree_.children_left == TREE_LEAF
+    assert_array_almost_equal(np.dot(node_indicator, all_leaves),
+                              np.ones(shape=n_samples))
+
+    # Ensure max depth is consistent with sum of indicator
+    max_depth = node_indicator.sum(axis=1).max()
+    assert_less_equal(est.tree_.max_depth, max_depth)
+
+
+def test_decision_path():
+    for name in ALL_TREES:
+        yield (check_decision_path, name)
+
+
+def check_no_sparse_y_support(name):
+    X, y = X_multilabel, csr_matrix(y_multilabel)
+    TreeEstimator = ALL_TREES[name]
+    assert_raises(TypeError, TreeEstimator(random_state=0).fit, X, y)
+
+
+def test_no_sparse_y_support():
+    # Currently we don't support sparse y
+    for name in ALL_TREES:
+        yield (check_no_sparse_y_support, name)
+
+def test_mae():
+    # check MAE criterion produces correct results
+    # on small toy dataset
+    dt_mae = DecisionTreeRegressor(random_state=0, criterion="mae",
+                                   max_leaf_nodes=2)
+    dt_mae.fit([[3],[5],[3],[8],[5]],[6,7,3,4,3])
+    assert_array_equal(dt_mae.tree_.impurity, [1.4, 1.5, 4.0/3.0])
+    assert_array_equal(dt_mae.tree_.value.flat, [4, 4.5, 4.0])
+
+    dt_mae.fit([[3],[5],[3],[8],[5]],[6,7,3,4,3], [0.6,0.3,0.1,1.0,0.3])
+    assert_array_equal(dt_mae.tree_.impurity, [7.0/2.3, 3.0/0.7, 4.0/1.6])
+    assert_array_equal(dt_mae.tree_.value.flat, [4.0, 6.0, 4.0])
