@@ -1,5 +1,3 @@
-import numpy as np
-from numpy.testing import TestCase
 from sklearn.utils.testing import assert_array_equal
 
 from sklearn.utils.stats import rankdata
